angular.module('cgNotify', []).factory('notify',['$timeout','$http','$compile','$templateCache','$rootScope',
    function($timeout,$http,$compile,$templateCache,$rootScope){

        var startTop = 10;
        var verticalSpacing = 15;
        var defaultDuration = 10000;
        var defaultTemplateUrl = 'angular-notify.html';
        var position = 'center';
        var container = document.body;
        var maximumOpen = 0;

        var messageElements = [];
        var openNotificationsScope = [];

        var notify = function(args){

            if (typeof args !== 'object'){
                args = {message:args};
            }

            args.duration = args.duration ? args.duration : defaultDuration;
            args.templateUrl = args.templateUrl ? args.templateUrl : defaultTemplateUrl;
            args.container = args.container ? args.container : container;
            args.classes = args.classes ? args.classes : '';

            var scope = args.scope ? args.scope.$new() : $rootScope.$new();
            scope.$position = args.position ? args.position : position;
            scope.$message = args.message;
            scope.$classes = args.classes;
            scope.$messageTemplate = args.messageTemplate;

            if (maximumOpen > 0) {
                var numToClose = (openNotificationsScope.length + 1) - maximumOpen;
                for (var i = 0; i < numToClose; i++) {
                    openNotificationsScope[i].$close();
                }
            }

            $http.get(args.templateUrl,{cache: $templateCache}).then(function(template){

                var templateElement = $compile(template.data)(scope);
                templateElement.bind('webkitTransitionEnd oTransitionEnd otransitionend transitionend msTransitionEnd', function(e){
                    if (e.propertyName === 'opacity' || e.currentTarget.style.opacity === 0 || 
                        (e.originalEvent && e.originalEvent.propertyName === 'opacity')){

                        templateElement.remove();
                        messageElements.splice(messageElements.indexOf(templateElement),1);
                        openNotificationsScope.splice(openNotificationsScope.indexOf(scope),1);
                        layoutMessages();
                    }
                });

                if (args.messageTemplate){
                    var messageTemplateElement;
                    for (var i = 0; i < templateElement.children().length; i ++){
                        if (angular.element(templateElement.children()[i]).hasClass('cg-notify-message-template')){
                            messageTemplateElement = angular.element(templateElement.children()[i]);
                            break;
                        }
                    }
                    if (messageTemplateElement){
                        messageTemplateElement.append($compile(args.messageTemplate)(scope));
                    } else {
                        throw new Error('cgNotify could not find the .cg-notify-message-template element in '+args.templateUrl+'.');
                    }
                }

                angular.element(args.container).append(templateElement);
                messageElements.push(templateElement);

                if (scope.$position === 'center'){
                    $timeout(function(){
                        scope.$centerMargin = '-' + (templateElement[0].offsetWidth /2) + 'px';
                    });
                }

                scope.$close = function(){
                    templateElement.css('opacity',0).attr('data-closing','true');
                    layoutMessages();
                };

                var layoutMessages = function(){
                    var j = 0;
                    var currentY = startTop;
                    for(var i = messageElements.length - 1; i >= 0; i --){
                        var shadowHeight = 10;
                        var element = messageElements[i];
                        var height = element[0].offsetHeight;
                        var top = currentY + height + shadowHeight;
                        if (element.attr('data-closing')){
                            top += 20;
                        } else {
                            currentY += height + verticalSpacing;
                        }
                        element.css('top',top + 'px').css('margin-top','-' + (height+shadowHeight) + 'px').css('visibility','visible');
                        j ++;
                    }
                };

                $timeout(function(){
                    layoutMessages();
                });

                if (args.duration > 0){
                    $timeout(function(){
                        scope.$close();
                    },args.duration);
                }

            }, function(data) {
                    throw new Error('Template specified for cgNotify ('+args.templateUrl+') could not be loaded. ' + data);
            });

            var retVal = {};
            
            retVal.close = function(){
                if (scope.$close){
                    scope.$close();
                }
            };

            Object.defineProperty(retVal,'message',{
                get: function(){
                    return scope.$message;
                },
                set: function(val){
                    scope.$message = val;
                }
            });

            openNotificationsScope.push(scope);

            return retVal;

        };

        notify.config = function(args){
            startTop = !angular.isUndefined(args.startTop) ? args.startTop : startTop;
            verticalSpacing = !angular.isUndefined(args.verticalSpacing) ? args.verticalSpacing : verticalSpacing;
            defaultDuration = !angular.isUndefined(args.duration) ? args.duration : defaultDuration;
            defaultTemplateUrl = args.templateUrl ? args.templateUrl : defaultTemplateUrl;
            position = !angular.isUndefined(args.position) ? args.position : position;
            container = args.container ? args.container : container;
            maximumOpen = args.maximumOpen ? args.maximumOpen : maximumOpen;
        };

        notify.closeAll = function(){
            for(var i = messageElements.length - 1; i >= 0; i --){
                var element = messageElements[i];
                element.css('opacity',0);
            }
        };

        return notify;
    }
]);

angular.module('cgNotify').run(['$templateCache', function($templateCache) {
  'use strict';

  $templateCache.put('angular-notify.html',
    "<div class=\"cg-notify-message\" ng-class=\"[$classes, \n" +
    "    $position === 'center' ? 'cg-notify-message-center' : '',\n" +
    "    $position === 'left' ? 'cg-notify-message-left' : '',\n" +
    "    $position === 'right' ? 'cg-notify-message-right' : '']\"\n" +
    "    ng-style=\"{'margin-left': $centerMargin}\">\n" +
    "\n" +
    "    <div ng-show=\"!$messageTemplate\">\n" +
    "        {{$message}}\n" +
    "    </div>\n" +
    "\n" +
    "    <div ng-show=\"$messageTemplate\" class=\"cg-notify-message-template\">\n" +
    "        \n" +
    "    </div>\n" +
    "\n" +
    "    <button type=\"button\" class=\"cg-notify-close\" ng-click=\"$close()\">\n" +
    "        <span aria-hidden=\"true\">&times;</span>\n" +
    "        <span class=\"cg-notify-sr-only\">Close</span>\n" +
    "    </button>\n" +
    "\n" +
    "</div>"
  );

}]);

/*!
 * clipboard.js v2.0.0
 * https://zenorocha.github.io/clipboard.js
 * 
 * Licensed MIT © Zeno Rocha
 */
(function webpackUniversalModuleDefinition(root, factory) {
	if(typeof exports === 'object' && typeof module === 'object')
		module.exports = factory();
	else if(typeof define === 'function' && define.amd)
		define([], factory);
	else if(typeof exports === 'object')
		exports["ClipboardJS"] = factory();
	else
		root["ClipboardJS"] = factory();
})(this, function() {
return /******/ (function(modules) { // webpackBootstrap
/******/ 	// The module cache
/******/ 	var installedModules = {};
/******/
/******/ 	// The require function
/******/ 	function __webpack_require__(moduleId) {
/******/
/******/ 		// Check if module is in cache
/******/ 		if(installedModules[moduleId]) {
/******/ 			return installedModules[moduleId].exports;
/******/ 		}
/******/ 		// Create a new module (and put it into the cache)
/******/ 		var module = installedModules[moduleId] = {
/******/ 			i: moduleId,
/******/ 			l: false,
/******/ 			exports: {}
/******/ 		};
/******/
/******/ 		// Execute the module function
/******/ 		modules[moduleId].call(module.exports, module, module.exports, __webpack_require__);
/******/
/******/ 		// Flag the module as loaded
/******/ 		module.l = true;
/******/
/******/ 		// Return the exports of the module
/******/ 		return module.exports;
/******/ 	}
/******/
/******/
/******/ 	// expose the modules object (__webpack_modules__)
/******/ 	__webpack_require__.m = modules;
/******/
/******/ 	// expose the module cache
/******/ 	__webpack_require__.c = installedModules;
/******/
/******/ 	// identity function for calling harmony imports with the correct context
/******/ 	__webpack_require__.i = function(value) { return value; };
/******/
/******/ 	// define getter function for harmony exports
/******/ 	__webpack_require__.d = function(exports, name, getter) {
/******/ 		if(!__webpack_require__.o(exports, name)) {
/******/ 			Object.defineProperty(exports, name, {
/******/ 				configurable: false,
/******/ 				enumerable: true,
/******/ 				get: getter
/******/ 			});
/******/ 		}
/******/ 	};
/******/
/******/ 	// getDefaultExport function for compatibility with non-harmony modules
/******/ 	__webpack_require__.n = function(module) {
/******/ 		var getter = module && module.__esModule ?
/******/ 			function getDefault() { return module['default']; } :
/******/ 			function getModuleExports() { return module; };
/******/ 		__webpack_require__.d(getter, 'a', getter);
/******/ 		return getter;
/******/ 	};
/******/
/******/ 	// Object.prototype.hasOwnProperty.call
/******/ 	__webpack_require__.o = function(object, property) { return Object.prototype.hasOwnProperty.call(object, property); };
/******/
/******/ 	// __webpack_public_path__
/******/ 	__webpack_require__.p = "";
/******/
/******/ 	// Load entry module and return exports
/******/ 	return __webpack_require__(__webpack_require__.s = 3);
/******/ })
/************************************************************************/
/******/ ([
/* 0 */
/***/ (function(module, exports, __webpack_require__) {

var __WEBPACK_AMD_DEFINE_FACTORY__, __WEBPACK_AMD_DEFINE_ARRAY__, __WEBPACK_AMD_DEFINE_RESULT__;(function (global, factory) {
    if (true) {
        !(__WEBPACK_AMD_DEFINE_ARRAY__ = [module, __webpack_require__(7)], __WEBPACK_AMD_DEFINE_FACTORY__ = (factory),
				__WEBPACK_AMD_DEFINE_RESULT__ = (typeof __WEBPACK_AMD_DEFINE_FACTORY__ === 'function' ?
				(__WEBPACK_AMD_DEFINE_FACTORY__.apply(exports, __WEBPACK_AMD_DEFINE_ARRAY__)) : __WEBPACK_AMD_DEFINE_FACTORY__),
				__WEBPACK_AMD_DEFINE_RESULT__ !== undefined && (module.exports = __WEBPACK_AMD_DEFINE_RESULT__));
    } else if (typeof exports !== "undefined") {
        factory(module, require('select'));
    } else {
        var mod = {
            exports: {}
        };
        factory(mod, global.select);
        global.clipboardAction = mod.exports;
    }
})(this, function (module, _select) {
    'use strict';

    var _select2 = _interopRequireDefault(_select);

    function _interopRequireDefault(obj) {
        return obj && obj.__esModule ? obj : {
            default: obj
        };
    }

    var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
        return typeof obj;
    } : function (obj) {
        return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };

    function _classCallCheck(instance, Constructor) {
        if (!(instance instanceof Constructor)) {
            throw new TypeError("Cannot call a class as a function");
        }
    }

    var _createClass = function () {
        function defineProperties(target, props) {
            for (var i = 0; i < props.length; i++) {
                var descriptor = props[i];
                descriptor.enumerable = descriptor.enumerable || false;
                descriptor.configurable = true;
                if ("value" in descriptor) descriptor.writable = true;
                Object.defineProperty(target, descriptor.key, descriptor);
            }
        }

        return function (Constructor, protoProps, staticProps) {
            if (protoProps) defineProperties(Constructor.prototype, protoProps);
            if (staticProps) defineProperties(Constructor, staticProps);
            return Constructor;
        };
    }();

    var ClipboardAction = function () {
        /**
         * @param {Object} options
         */
        function ClipboardAction(options) {
            _classCallCheck(this, ClipboardAction);

            this.resolveOptions(options);
            this.initSelection();
        }

        /**
         * Defines base properties passed from constructor.
         * @param {Object} options
         */


        _createClass(ClipboardAction, [{
            key: 'resolveOptions',
            value: function resolveOptions() {
                var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

                this.action = options.action;
                this.container = options.container;
                this.emitter = options.emitter;
                this.target = options.target;
                this.text = options.text;
                this.trigger = options.trigger;

                this.selectedText = '';
            }
        }, {
            key: 'initSelection',
            value: function initSelection() {
                if (this.text) {
                    this.selectFake();
                } else if (this.target) {
                    this.selectTarget();
                }
            }
        }, {
            key: 'selectFake',
            value: function selectFake() {
                var _this = this;

                var isRTL = document.documentElement.getAttribute('dir') == 'rtl';

                this.removeFake();

                this.fakeHandlerCallback = function () {
                    return _this.removeFake();
                };
                this.fakeHandler = this.container.addEventListener('click', this.fakeHandlerCallback) || true;

                this.fakeElem = document.createElement('textarea');
                // Prevent zooming on iOS
                this.fakeElem.style.fontSize = '12pt';
                // Reset box model
                this.fakeElem.style.border = '0';
                this.fakeElem.style.padding = '0';
                this.fakeElem.style.margin = '0';
                // Move element out of screen horizontally
                this.fakeElem.style.position = 'absolute';
                this.fakeElem.style[isRTL ? 'right' : 'left'] = '-9999px';
                // Move element to the same position vertically
                var yPosition = window.pageYOffset || document.documentElement.scrollTop;
                this.fakeElem.style.top = yPosition + 'px';

                this.fakeElem.setAttribute('readonly', '');
                this.fakeElem.value = this.text;

                this.container.appendChild(this.fakeElem);

                this.selectedText = (0, _select2.default)(this.fakeElem);
                this.copyText();
            }
        }, {
            key: 'removeFake',
            value: function removeFake() {
                if (this.fakeHandler) {
                    this.container.removeEventListener('click', this.fakeHandlerCallback);
                    this.fakeHandler = null;
                    this.fakeHandlerCallback = null;
                }

                if (this.fakeElem) {
                    this.container.removeChild(this.fakeElem);
                    this.fakeElem = null;
                }
            }
        }, {
            key: 'selectTarget',
            value: function selectTarget() {
                this.selectedText = (0, _select2.default)(this.target);
                this.copyText();
            }
        }, {
            key: 'copyText',
            value: function copyText() {
                var succeeded = void 0;

                try {
                    succeeded = document.execCommand(this.action);
                } catch (err) {
                    succeeded = false;
                }

                this.handleResult(succeeded);
            }
        }, {
            key: 'handleResult',
            value: function handleResult(succeeded) {
                this.emitter.emit(succeeded ? 'success' : 'error', {
                    action: this.action,
                    text: this.selectedText,
                    trigger: this.trigger,
                    clearSelection: this.clearSelection.bind(this)
                });
            }
        }, {
            key: 'clearSelection',
            value: function clearSelection() {
                if (this.trigger) {
                    this.trigger.focus();
                }

                window.getSelection().removeAllRanges();
            }
        }, {
            key: 'destroy',
            value: function destroy() {
                this.removeFake();
            }
        }, {
            key: 'action',
            set: function set() {
                var action = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'copy';

                this._action = action;

                if (this._action !== 'copy' && this._action !== 'cut') {
                    throw new Error('Invalid "action" value, use either "copy" or "cut"');
                }
            },
            get: function get() {
                return this._action;
            }
        }, {
            key: 'target',
            set: function set(target) {
                if (target !== undefined) {
                    if (target && (typeof target === 'undefined' ? 'undefined' : _typeof(target)) === 'object' && target.nodeType === 1) {
                        if (this.action === 'copy' && target.hasAttribute('disabled')) {
                            throw new Error('Invalid "target" attribute. Please use "readonly" instead of "disabled" attribute');
                        }

                        if (this.action === 'cut' && (target.hasAttribute('readonly') || target.hasAttribute('disabled'))) {
                            throw new Error('Invalid "target" attribute. You can\'t cut text from elements with "readonly" or "disabled" attributes');
                        }

                        this._target = target;
                    } else {
                        throw new Error('Invalid "target" value, use a valid Element');
                    }
                }
            },
            get: function get() {
                return this._target;
            }
        }]);

        return ClipboardAction;
    }();

    module.exports = ClipboardAction;
});

/***/ }),
/* 1 */
/***/ (function(module, exports, __webpack_require__) {

var is = __webpack_require__(6);
var delegate = __webpack_require__(5);

/**
 * Validates all params and calls the right
 * listener function based on its target type.
 *
 * @param {String|HTMLElement|HTMLCollection|NodeList} target
 * @param {String} type
 * @param {Function} callback
 * @return {Object}
 */
function listen(target, type, callback) {
    if (!target && !type && !callback) {
        throw new Error('Missing required arguments');
    }

    if (!is.string(type)) {
        throw new TypeError('Second argument must be a String');
    }

    if (!is.fn(callback)) {
        throw new TypeError('Third argument must be a Function');
    }

    if (is.node(target)) {
        return listenNode(target, type, callback);
    }
    else if (is.nodeList(target)) {
        return listenNodeList(target, type, callback);
    }
    else if (is.string(target)) {
        return listenSelector(target, type, callback);
    }
    else {
        throw new TypeError('First argument must be a String, HTMLElement, HTMLCollection, or NodeList');
    }
}

/**
 * Adds an event listener to a HTML element
 * and returns a remove listener function.
 *
 * @param {HTMLElement} node
 * @param {String} type
 * @param {Function} callback
 * @return {Object}
 */
function listenNode(node, type, callback) {
    node.addEventListener(type, callback);

    return {
        destroy: function() {
            node.removeEventListener(type, callback);
        }
    }
}

/**
 * Add an event listener to a list of HTML elements
 * and returns a remove listener function.
 *
 * @param {NodeList|HTMLCollection} nodeList
 * @param {String} type
 * @param {Function} callback
 * @return {Object}
 */
function listenNodeList(nodeList, type, callback) {
    Array.prototype.forEach.call(nodeList, function(node) {
        node.addEventListener(type, callback);
    });

    return {
        destroy: function() {
            Array.prototype.forEach.call(nodeList, function(node) {
                node.removeEventListener(type, callback);
            });
        }
    }
}

/**
 * Add an event listener to a selector
 * and returns a remove listener function.
 *
 * @param {String} selector
 * @param {String} type
 * @param {Function} callback
 * @return {Object}
 */
function listenSelector(selector, type, callback) {
    return delegate(document.body, selector, type, callback);
}

module.exports = listen;


/***/ }),
/* 2 */
/***/ (function(module, exports) {

function E () {
  // Keep this empty so it's easier to inherit from
  // (via https://github.com/lipsmack from https://github.com/scottcorgan/tiny-emitter/issues/3)
}

E.prototype = {
  on: function (name, callback, ctx) {
    var e = this.e || (this.e = {});

    (e[name] || (e[name] = [])).push({
      fn: callback,
      ctx: ctx
    });

    return this;
  },

  once: function (name, callback, ctx) {
    var self = this;
    function listener () {
      self.off(name, listener);
      callback.apply(ctx, arguments);
    };

    listener._ = callback
    return this.on(name, listener, ctx);
  },

  emit: function (name) {
    var data = [].slice.call(arguments, 1);
    var evtArr = ((this.e || (this.e = {}))[name] || []).slice();
    var i = 0;
    var len = evtArr.length;

    for (i; i < len; i++) {
      evtArr[i].fn.apply(evtArr[i].ctx, data);
    }

    return this;
  },

  off: function (name, callback) {
    var e = this.e || (this.e = {});
    var evts = e[name];
    var liveEvents = [];

    if (evts && callback) {
      for (var i = 0, len = evts.length; i < len; i++) {
        if (evts[i].fn !== callback && evts[i].fn._ !== callback)
          liveEvents.push(evts[i]);
      }
    }

    // Remove event from queue to prevent memory leak
    // Suggested by https://github.com/lazd
    // Ref: https://github.com/scottcorgan/tiny-emitter/commit/c6ebfaa9bc973b33d110a84a307742b7cf94c953#commitcomment-5024910

    (liveEvents.length)
      ? e[name] = liveEvents
      : delete e[name];

    return this;
  }
};

module.exports = E;


/***/ }),
/* 3 */
/***/ (function(module, exports, __webpack_require__) {

var __WEBPACK_AMD_DEFINE_FACTORY__, __WEBPACK_AMD_DEFINE_ARRAY__, __WEBPACK_AMD_DEFINE_RESULT__;(function (global, factory) {
    if (true) {
        !(__WEBPACK_AMD_DEFINE_ARRAY__ = [module, __webpack_require__(0), __webpack_require__(2), __webpack_require__(1)], __WEBPACK_AMD_DEFINE_FACTORY__ = (factory),
				__WEBPACK_AMD_DEFINE_RESULT__ = (typeof __WEBPACK_AMD_DEFINE_FACTORY__ === 'function' ?
				(__WEBPACK_AMD_DEFINE_FACTORY__.apply(exports, __WEBPACK_AMD_DEFINE_ARRAY__)) : __WEBPACK_AMD_DEFINE_FACTORY__),
				__WEBPACK_AMD_DEFINE_RESULT__ !== undefined && (module.exports = __WEBPACK_AMD_DEFINE_RESULT__));
    } else if (typeof exports !== "undefined") {
        factory(module, require('./clipboard-action'), require('tiny-emitter'), require('good-listener'));
    } else {
        var mod = {
            exports: {}
        };
        factory(mod, global.clipboardAction, global.tinyEmitter, global.goodListener);
        global.clipboard = mod.exports;
    }
})(this, function (module, _clipboardAction, _tinyEmitter, _goodListener) {
    'use strict';

    var _clipboardAction2 = _interopRequireDefault(_clipboardAction);

    var _tinyEmitter2 = _interopRequireDefault(_tinyEmitter);

    var _goodListener2 = _interopRequireDefault(_goodListener);

    function _interopRequireDefault(obj) {
        return obj && obj.__esModule ? obj : {
            default: obj
        };
    }

    var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
        return typeof obj;
    } : function (obj) {
        return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };

    function _classCallCheck(instance, Constructor) {
        if (!(instance instanceof Constructor)) {
            throw new TypeError("Cannot call a class as a function");
        }
    }

    var _createClass = function () {
        function defineProperties(target, props) {
            for (var i = 0; i < props.length; i++) {
                var descriptor = props[i];
                descriptor.enumerable = descriptor.enumerable || false;
                descriptor.configurable = true;
                if ("value" in descriptor) descriptor.writable = true;
                Object.defineProperty(target, descriptor.key, descriptor);
            }
        }

        return function (Constructor, protoProps, staticProps) {
            if (protoProps) defineProperties(Constructor.prototype, protoProps);
            if (staticProps) defineProperties(Constructor, staticProps);
            return Constructor;
        };
    }();

    function _possibleConstructorReturn(self, call) {
        if (!self) {
            throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
        }

        return call && (typeof call === "object" || typeof call === "function") ? call : self;
    }

    function _inherits(subClass, superClass) {
        if (typeof superClass !== "function" && superClass !== null) {
            throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
        }

        subClass.prototype = Object.create(superClass && superClass.prototype, {
            constructor: {
                value: subClass,
                enumerable: false,
                writable: true,
                configurable: true
            }
        });
        if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
    }

    var Clipboard = function (_Emitter) {
        _inherits(Clipboard, _Emitter);

        /**
         * @param {String|HTMLElement|HTMLCollection|NodeList} trigger
         * @param {Object} options
         */
        function Clipboard(trigger, options) {
            _classCallCheck(this, Clipboard);

            var _this = _possibleConstructorReturn(this, (Clipboard.__proto__ || Object.getPrototypeOf(Clipboard)).call(this));

            _this.resolveOptions(options);
            _this.listenClick(trigger);
            return _this;
        }

        /**
         * Defines if attributes would be resolved using internal setter functions
         * or custom functions that were passed in the constructor.
         * @param {Object} options
         */


        _createClass(Clipboard, [{
            key: 'resolveOptions',
            value: function resolveOptions() {
                var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

                this.action = typeof options.action === 'function' ? options.action : this.defaultAction;
                this.target = typeof options.target === 'function' ? options.target : this.defaultTarget;
                this.text = typeof options.text === 'function' ? options.text : this.defaultText;
                this.container = _typeof(options.container) === 'object' ? options.container : document.body;
            }
        }, {
            key: 'listenClick',
            value: function listenClick(trigger) {
                var _this2 = this;

                this.listener = (0, _goodListener2.default)(trigger, 'click', function (e) {
                    return _this2.onClick(e);
                });
            }
        }, {
            key: 'onClick',
            value: function onClick(e) {
                var trigger = e.delegateTarget || e.currentTarget;

                if (this.clipboardAction) {
                    this.clipboardAction = null;
                }

                this.clipboardAction = new _clipboardAction2.default({
                    action: this.action(trigger),
                    target: this.target(trigger),
                    text: this.text(trigger),
                    container: this.container,
                    trigger: trigger,
                    emitter: this
                });
            }
        }, {
            key: 'defaultAction',
            value: function defaultAction(trigger) {
                return getAttributeValue('action', trigger);
            }
        }, {
            key: 'defaultTarget',
            value: function defaultTarget(trigger) {
                var selector = getAttributeValue('target', trigger);

                if (selector) {
                    return document.querySelector(selector);
                }
            }
        }, {
            key: 'defaultText',
            value: function defaultText(trigger) {
                return getAttributeValue('text', trigger);
            }
        }, {
            key: 'destroy',
            value: function destroy() {
                this.listener.destroy();

                if (this.clipboardAction) {
                    this.clipboardAction.destroy();
                    this.clipboardAction = null;
                }
            }
        }], [{
            key: 'isSupported',
            value: function isSupported() {
                var action = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : ['copy', 'cut'];

                var actions = typeof action === 'string' ? [action] : action;
                var support = !!document.queryCommandSupported;

                actions.forEach(function (action) {
                    support = support && !!document.queryCommandSupported(action);
                });

                return support;
            }
        }]);

        return Clipboard;
    }(_tinyEmitter2.default);

    /**
     * Helper function to retrieve attribute value.
     * @param {String} suffix
     * @param {Element} element
     */
    function getAttributeValue(suffix, element) {
        var attribute = 'data-clipboard-' + suffix;

        if (!element.hasAttribute(attribute)) {
            return;
        }

        return element.getAttribute(attribute);
    }

    module.exports = Clipboard;
});

/***/ }),
/* 4 */
/***/ (function(module, exports) {

var DOCUMENT_NODE_TYPE = 9;

/**
 * A polyfill for Element.matches()
 */
if (typeof Element !== 'undefined' && !Element.prototype.matches) {
    var proto = Element.prototype;

    proto.matches = proto.matchesSelector ||
                    proto.mozMatchesSelector ||
                    proto.msMatchesSelector ||
                    proto.oMatchesSelector ||
                    proto.webkitMatchesSelector;
}

/**
 * Finds the closest parent that matches a selector.
 *
 * @param {Element} element
 * @param {String} selector
 * @return {Function}
 */
function closest (element, selector) {
    while (element && element.nodeType !== DOCUMENT_NODE_TYPE) {
        if (typeof element.matches === 'function' &&
            element.matches(selector)) {
          return element;
        }
        element = element.parentNode;
    }
}

module.exports = closest;


/***/ }),
/* 5 */
/***/ (function(module, exports, __webpack_require__) {

var closest = __webpack_require__(4);

/**
 * Delegates event to a selector.
 *
 * @param {Element} element
 * @param {String} selector
 * @param {String} type
 * @param {Function} callback
 * @param {Boolean} useCapture
 * @return {Object}
 */
function _delegate(element, selector, type, callback, useCapture) {
    var listenerFn = listener.apply(this, arguments);

    element.addEventListener(type, listenerFn, useCapture);

    return {
        destroy: function() {
            element.removeEventListener(type, listenerFn, useCapture);
        }
    }
}

/**
 * Delegates event to a selector.
 *
 * @param {Element|String|Array} [elements]
 * @param {String} selector
 * @param {String} type
 * @param {Function} callback
 * @param {Boolean} useCapture
 * @return {Object}
 */
function delegate(elements, selector, type, callback, useCapture) {
    // Handle the regular Element usage
    if (typeof elements.addEventListener === 'function') {
        return _delegate.apply(null, arguments);
    }

    // Handle Element-less usage, it defaults to global delegation
    if (typeof type === 'function') {
        // Use `document` as the first parameter, then apply arguments
        // This is a short way to .unshift `arguments` without running into deoptimizations
        return _delegate.bind(null, document).apply(null, arguments);
    }

    // Handle Selector-based usage
    if (typeof elements === 'string') {
        elements = document.querySelectorAll(elements);
    }

    // Handle Array-like based usage
    return Array.prototype.map.call(elements, function (element) {
        return _delegate(element, selector, type, callback, useCapture);
    });
}

/**
 * Finds closest match and invokes callback.
 *
 * @param {Element} element
 * @param {String} selector
 * @param {String} type
 * @param {Function} callback
 * @return {Function}
 */
function listener(element, selector, type, callback) {
    return function(e) {
        e.delegateTarget = closest(e.target, selector);

        if (e.delegateTarget) {
            callback.call(element, e);
        }
    }
}

module.exports = delegate;


/***/ }),
/* 6 */
/***/ (function(module, exports) {

/**
 * Check if argument is a HTML element.
 *
 * @param {Object} value
 * @return {Boolean}
 */
exports.node = function(value) {
    return value !== undefined
        && value instanceof HTMLElement
        && value.nodeType === 1;
};

/**
 * Check if argument is a list of HTML elements.
 *
 * @param {Object} value
 * @return {Boolean}
 */
exports.nodeList = function(value) {
    var type = Object.prototype.toString.call(value);

    return value !== undefined
        && (type === '[object NodeList]' || type === '[object HTMLCollection]')
        && ('length' in value)
        && (value.length === 0 || exports.node(value[0]));
};

/**
 * Check if argument is a string.
 *
 * @param {Object} value
 * @return {Boolean}
 */
exports.string = function(value) {
    return typeof value === 'string'
        || value instanceof String;
};

/**
 * Check if argument is a function.
 *
 * @param {Object} value
 * @return {Boolean}
 */
exports.fn = function(value) {
    var type = Object.prototype.toString.call(value);

    return type === '[object Function]';
};


/***/ }),
/* 7 */
/***/ (function(module, exports) {

function select(element) {
    var selectedText;

    if (element.nodeName === 'SELECT') {
        element.focus();

        selectedText = element.value;
    }
    else if (element.nodeName === 'INPUT' || element.nodeName === 'TEXTAREA') {
        var isReadOnly = element.hasAttribute('readonly');

        if (!isReadOnly) {
            element.setAttribute('readonly', '');
        }

        element.select();
        element.setSelectionRange(0, element.value.length);

        if (!isReadOnly) {
            element.removeAttribute('readonly');
        }

        selectedText = element.value;
    }
    else {
        if (element.hasAttribute('contenteditable')) {
            element.focus();
        }

        var selection = window.getSelection();
        var range = document.createRange();

        range.selectNodeContents(element);
        selection.removeAllRanges();
        selection.addRange(range);

        selectedText = selection.toString();
    }

    return selectedText;
}

module.exports = select;


/***/ })
/******/ ]);
});
/*! ngclipboard - v2.0.0 - 2018-03-03
* https://github.com/sachinchoolur/ngclipboard
* Copyright (c) 2018 Sachin; Licensed MIT */
(function() {
    'use strict';
    var MODULE_NAME = 'ngclipboard';
    var angular, ClipboardJS;

    // Check for CommonJS support
    if (typeof module === 'object' && module.exports) {
      angular = require('angular');
      ClipboardJS = require('clipboard');
      module.exports = MODULE_NAME;
    } else {
      angular = window.angular;
      ClipboardJS = window.ClipboardJS;
    }

    angular.module(MODULE_NAME, []).directive('ngclipboard', function() {
        return {
            restrict: 'A',
            scope: {
                ngclipboardSuccess: '&',
                ngclipboardError: '&'
            },
            link: function(scope, element) {
                //constructor for clipboardjs changed to ClipboardJS
                var clipboard = new ClipboardJS(element[0]);

                clipboard.on('success', function(e) {
                  scope.$apply(function () {
                    scope.ngclipboardSuccess({
                      e: e
                    });
                  });
                });

                clipboard.on('error', function(e) {
                  scope.$apply(function () {
                    scope.ngclipboardError({
                      e: e
                    });
                  });
                });

                element.on('$destroy', function() {
                    clipboard.destroy();
                });

            }
        };
    });
})();

;angular.module('Landers.angular', [
    'cgNotify',
    'ngclipboard',
])
.run(['$rootScope', function ($rootScope){
    $rootScope.$GEvent = {
        // 剪切板
        clipboard: {
            showTooltip: function(e, title){
                $(e.trigger)
                    .tooltip({
                        title: title,
                    })
                    .tooltip('show')
                    .mouseout(function(){
                        $(this).tooltip('destroy');
                    });
            },
            success: function (e) {
                this.showTooltip(e, '复制成功');
            },
            error: function (e) {
                this.showTooltip(e, '复制失败');
            }
        }
    }
}]);
function angular_scope_apply(scope) {
    if (!scope.$$phase ) {
        if (!scope.hasOwnProperty('$$preventApply') && !scope.$$preventApply) {
            try {
                // console.log('angular_scope_apply');
                scope.$apply();
            } catch(e) {

            }
        }
    }
};
if (window.$) {
    $.fn.attachWatcher = function(fn) {
        $(this).data('angularDestroyWatch', fn);
    };
    $.fn.destroyWatch = function(){
        var fn = $(this).data('angularDestroyWatch');
        if (fn) fn();
        return this;
    };
}
;angular.module('Landers.angular')
    .directive('includes', ['$compile', function($compile){
        return {
            restrict : 'A',
            link : function($scope, $ele, attrs) {
                var includes = attrs['includes'].split('|');
                var tpl = '<div ng-include src="$partial(\'{mod}\')"></div>';
                var tpls = [];
                angular.forEach(includes, function(item){
                    tpls.push(tpl.replace('{mod}', item));
                })
                var html = tpls.join('\n');
                $ele.children().html($compile(html)($scope));
            }
        }
    }]);
;angular.module('Landers.angular')
    .directive('keyToText', ['$timeout', 'Flat', function($timeout, Flat) {
        return {
            restrict : 'E',
            scope: {
                ngModel: '=',
                enums: '='
            },
            link : function($scope, $ele, $attrs) {
                var enums = $scope.enums;
                var default_value = $attrs['defaultValue'] || ('defaultValue' in $attrs);
                var field = $attrs['key'] || 'text';
                $scope.$watch('ngModel', function(new_value){
                    if (new_value === undefined || new_value === null) return;
                    var data = enums[new_value];
                    if (!data && default_value) {
                        data = {};
                        data[field] = default_value === true ? new_value : default_value;
                        data._class = $attrs['defaultClass'];
                    }
                    if (!data) return '';

                    $ele.html(data[field]);

                    if (data._class) {
                        for (var k in enums) $ele.removeClass(enums[k]['_class']);
                        $ele.addClass(data._class);
                    }
                });
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('toggleClasses', ['Flat', '$timeout', function(Flat, $timeout) {
        return {
            restrict : 'A',
            // require : '?ngModel',
            link : function($scope, $ele, $attrs) {
                var arr1 = $attrs['toggleClasses'].split(',');
                var arr2 = arr1[0].split('|'), callback = arr1[1];

                $timeout(function(){
                    if (callback) {
                        callback = Flat.get($scope, callback);
                    };
                    $ele.data('ToggleClassIndex', 0)
                    .addClass(arr2[0]).click(function(){
                        var last_index = $ele.data('ToggleClassIndex');
                        var index = last_index + 1;
                        if (index >= arr2.length) index = 0;
                        for (var i=0; i<arr2.length; i++) $ele.removeClass(arr2[i]);
                        $ele.data('ToggleClassIndex', index).addClass(arr2[index]);
                        callback && callback(index, arr2[index]);
                    });
                }, 400);

            }
        };
    }]);
;angular.module('Landers.angular')
    .factory('BaseService', ['Ajax', 'Runtime', function (Ajax, Runtime){
        function BaseService(scope, extend, events){
            this.ajax = Ajax.make();
            this.runtime = Runtime.make(scope);
            angular.extend(this, extend || {});
        }
        return {
            make:function(scope, extend, events) {
                return new BaseService(scope, extend, events);
            }
        };
    }]);
;angular.module('Landers.angular')
    .factory('DataConvert', function(){
        return {
            checkbox:function(xvar) {
                var ret;
                if (angular.isString(xvar)) {
                    xvar = angular.toJson(xvar);
                }
                if (angular.isArray(xvar)) {
                    ret = {};
                    angular.forEach(xvar, function(item){
                        ret[item] = true;
                    });
                } else {
                    if (angular.isObject(xvar)) {
                        ret = [];
                        for (var k in xvar) {
                            if (xvar[k]) ret.push(k);
                        }
                    }
                }
                return ret;
            }
        };
    });
;angular.module('Landers.angular')
    .factory('Flat', function () {
        function Flat(J, scope){
            var that = Flat;
            this.apply = function(){
                if ( scope && ('$$phase' in scope) && (!scope.$$phase)) {
                    angular_scope_apply(scope);
                }
                return this;
            };
            this.get = function(key, subkey) {
                return that.get(J, key, subkey);
            };
            this.pull = function(key) {
                var ret = that.pull(J, key);
                this.apply();
                return ret;
            };
            this.set = function(key, value, subkey){
                that.set(J, key, value, subkey);
                return this.apply();
            };
            this.setOnce = function(key, value, subkey){
                var ret = that.setOnce(J, key, value, subkey);
                this.apply();
                return ret;
            };
            this.delete = function(key) {
                that.delete(J, key);
                return this.apply();
            };
            this.clear = function(){
                that.clear(J);
                return this.apply();
            };
            this.extend = function(json, subkey){
                that.extend(J, json, subkey);
                return this.apply();
            };
            this.cut = function(key, index) {
                that.cut(J, key, index);
                return this.apply();
            };
            this.copy = function(key, start, end) {
                that.copy(J, key, start, end);
                return this.apply();
            };
            this.shift = function(key) {
                that.shift(J, key);
                return this.apply();
            };
            this.unshift = function(key, value) {
                that.unshift(J, key, value);
                return this.apply();
            };
            this.push = function(key, value){
                that.push(J, key, value);
                return this.apply();
            };
            this.pop = function(key) {
                that.pop(J, key);
                return this.apply();
            };
        }
        angular.extend(Flat, {
            make : function(J, scope) {
                return new Flat(J, scope);
            },
            parse : function(J, key, subkey){
                if (subkey !== null && subkey !== undefined && subkey !== '') key += '.' + subkey;
                var a = key.split('.');
                J = J || {};
                if (a.length == 1) {
                    return {
                        obj:J, key:key
                    };
                } else {
                    for (var i=0; i<a.length-1; i++) {
                        J = J || {};
                        if ( ! (a[i] in J) ) {
                            J[a[i]] = {};
                        }
                        J[a[i]] = J[a[i]] || {};
                        J = J[a[i]];
                    }
                    return {
                        obj:J,
                        key:a[i]
                    };
                }
            },
            /**
             * Object操作
             * @param  {[type]} J      [description]
             * @param  {[type]} key    [description]
             * @param  {[type]} subkey [description]
             * @return {[type]}        [description]
             */
            get : function(J, key, subkey) {
                var parse = this.parse(J, key, subkey);
                return parse.obj[parse.key];
            },
            pull : function(J, key){
                var ret = this.get(J, key);
                this.delete(J, key);
                return ret;
            },
            set : function(J, key, value, subkey){
                var parse = this.parse(J, key, subkey);
                // parse.obj = parse.obj || {}; //promote 为空时
                parse.obj[parse.key] = value;
                return this;
            },
            setOnce : function(J, key, value, subkey){
                var ret = this.get(J, key, subkey);
                if ( ret ) return ret;
                this.set(J, key, value, subkey);
                return value;
            },
            delete : function(J, key) {
                var parse = this.parse(J, key);
                if (angular.isArray(parse.obj)) {
                    var index = Number(parse.key);
                    if (isNaN(index)) {
                        throw 'parse.key: ' + parse.key + '为非数字';
                    }
                    parse.obj.splice(index, 1);
                } else {
                    delete parse.obj[parse.key];
                }
                return this;
            },
            clear : function(J) {
                for (var k in J) delete J[k];
                J = {};
                return this;
            },
            extend : function(J, jdat, subkey) {
                if (subkey) {
                    var value = this.get(J, subkey) || {};
                    angular.extend(value, jdat);
                    this.set(J, subkey, value);
                } else {
                    for (var key in jdat) {
                        this.set(J, key, jdat[key]);
                    }
                }
                return this;
            },
            cut : function(J, key, index) {
                var parse = this.parse(J, key);
                parse.obj[parse.key].splice(index, 1);
                return parse.obj[parse.key];
            },
            copy : function(J, key, start, end) {
                var parse = this.parse(J, key);
                return parse.obj[parse.key].slice(start, end);
            },

            /**
             * Array 操作
             * @param  {[type]} J     [description]
             * @param  {[type]} key   [description]
             * @param  {[type]} index [description]
             * @param  {[type]} dat   [description]
             * @return {[type]}       [description]
             */
            splice: function(J, key, index, dat){
                var parse = this.parse(J, key);
                if (dat) {
                    parse.obj.data.splice(index, 1, dat);
                } else {
                    parse.obj.data.splice(index, 1);
                }
            },
            shift : function(J, key) {
                var parse = this.parse(J, key);
                parse.obj[parse.key].shift();
                return this;
            },
            unshift : function(J, key, value) {
                var parse = this.parse(J, key);
                try {
                    parse.obj[parse.key].unshift(value);
                }catch(e){
                    console.log(J);
                    console.log(key);
                    console.log(value);
                }
                return this;
            },
            push : function(J, key, value){
                var parse = this.parse(J, key);
                parse.obj[parse.key] = parse.obj[parse.key] || [];
                parse.obj[parse.key].push(value);
                return this;
            },
            pop : function(J, key) {
                var parse = this.parse(J, key);
                parse.obj[parse.key].pop();
                return this;
            }
        });

        return Flat;
    });
;angular.module('Landers.angular')
    .factory('Helpers', ['Flat', '$timeout', function(Flat, $timeout){
        var cache = {};

        return {
            seekScope : function(scope, parentId) {
                var myScope = scope;
                if ( !parentId) return myScope;
                while (myScope.$id != parentId) {
                    if (myScope.$parent) {
                        myScope = myScope.$parent;
                    } else {
                        break;
                    }
                }
                return myScope;
            },
            toJqEle : function($event) {
                if (!$event) return $();
                var obj = angular.isElement($event) ? $event : $event.target;
                return angular.element(obj);
            },
            disable : function($event){
                $event = this.toJqEle($event);
                return Landers.helpers.disable($event);
            },
            enable : function($event) {
                $event = this.toJqEle($event);
                return Landers.helpers.enable($event);
            },
            domainUrl : function(domainUrl, url) {
                if (!domainUrl) return url;
                var len = domainUrl.length;
                var pre = Landers.utils.str.left(url, len);
                if (pre === domainUrl) return url;
                else return domainUrl + url;
            },
            timeouts: function(callbacks, delay){
                (function(i){
                    var callback = callbacks[i];
                    var self = arguments.callee;
                    $timeout(function(){
                        callback && callback();
                        self(++i);
                    }, delay);
                })(0);
            },
            attrItemToJson: function(attr, key) {
                var ret = attr[key] || {};
                return angular.isString(ret) ? angular.fromJson(ret.replace(/(')+/gm, '"')) : ret;
            },
            $on: {
                once: function(scope, event, callback) {
                    var key = [scope.id, event].join('.');
                    if (!Flat.get(cache, key)) {
                        Flat.set(cache, key, scope.$on(event, callback));
                    }
                    return this;
                },
                deregister: function(scope, event){
                    var key = [scope.id, event].join('.');
                    var fun = Flat.get(cache, key);
                    fun && fun();
                    Flat.set(cache, key, null);
                }
            }
        };
    }]);
;angular.module('Landers.angular')
    .factory('HttpInterceptor', ['$log', '$q', '$location', function($log, $q, $location) {
        return {
            response: function(response) {
                var deferred = $q.defer();
                deferred.resolve(response);
                return deferred.promise;
            },

            responseError : function(rejection){
                var deferred = $q.defer();
                deferred.reject(rejection);
                return deferred.promise;
            }
        };
    }]);
;angular.module('Landers.angular')
    .factory('InjectService', ['$ocLazyLoad', '$injector', function($ocLazyLoad, $injector){
        return {
            urls:function(urls, callback){
                $ocLazyLoad.load(urls).then(callback);
            },
            module : function(urls, injects) {
                if (!angular.isArray(injects)) {
                    injects = [injects];
                }

                this.urls(urls, function(){
                    $injector.invoke(injects);
                });
            }
        };
    }]);
;angular.module('Landers.angular')
    .factory('List', ['Flat', 'Helpers', function (Flat, Helpers) {
        var _class = {
            list:'list',
            head:'list-head',
            item:'list-item',
            checked:'checked',
            actived:'active',
            checker:'checker',
        };

        /**
         * 列表项类（目前定位操作dom为主）
         * @return {[type]} [description]
         */
        function Listitem($object) {
            var that = this;

            var check_object = function(){
                if (!that.object) throw '未设置列表项对象';
            };

            /**
             * 设置列表项对象
             * @param {[type]} $object [description]
             */
            this.setObject = function($object) {
                if ($object) {
                    that.object = $object;
                }
                return that;
            }

            /**
             * 取得列表项对象
             * @return {[type]} [description]
             */
            this.getObject = function(){
                return that.object;
            };

            /**
             * 通过事件触发源设置列表项对象
             * @param {[type]} $event [description]
             */
            this.setObjectByEvent = function($event) {
                if (!angular.isElement($event)) {
                    $event = angular.element($event.target);
                }

                that.object = $event.closest('.' + _class.item);
                return that;
            };

            this.getIndex = function() {
                check_object();
                return that.object.index();
            };

            /**
             * 取得列表对象
             * @return {[type]} [description]
             */
            this.getListObject = function() {
                check_object();
                return that.closest('.' + _class.list);
            };

            /**
             * 根据xvar(item)
             * 选中列表项
             * @param string checked_class
             * @return {[type]} [description]
             */
            this.checked = function(checked_class) {
                check_object();
                that.object.addClass(checked_class || _class.checked);
                return that;
            };

            /**
             * 根据xvar(item)
             * 取消选中列表项
             * @param string checked_class
             * @return {[type]} [description]
             */
            this.unchecked = function(checked_class) {
                check_object();
                that.object.removeClass(checked_class || _class.checked);
                return that;
            };

            /**
             * 根据xvar(item)
             * 选中列表项
             * @param string actived_class
             * @return {[type]} [description]
             */
            this.actived = function(actived_class) {
                check_object();
                that.object.addClass(actived_class || _class.actived);
                return that;
            };

            /**
             * 根据xvar(item)
             * 取消选中列表项
             * @param string actived_class
             * @return {[type]} [description]
             */
            this.unactived = function(actived_class) {
                check_object();
                that.object.removeClass(actived_class || _class.actived);
                return that;
            };

            that.setObject($object);
        };

        /**
         * 列表类
         * @param  {[type]} scope         [description]
         * @param  {[type]} datakey       [description]
         * @param  {[type]} extendMethods [description]
         * @return {[type]}               [description]
         */
        function list(scope, datakey, extendMethods) {
            var that = this;
            angular.extend(this, extendMethods);
            this.scope = scope;
            this.datakey = datakey;

            var flat = Flat.make(scope, scope);

            /**
             * 默认idKey
             * @type {String}
             */
            this.idKey = 'id';

            /**
             * 设置idKey
             * @param {[type]} idkey [description]
             */
            this.setIdKey = function(idkey){
                that.idKey = idkey;
            };

            /**
             * 设置数据
             * @param  {[type]} data [description]
             * @return {[type]}      [description]
             */
            this.put = function(data){
                if (angular.isString(data)) {
                    data = angular.fromJson(data);
                }
                flat.set(datakey, data);
                return that;
            };

            /**
             * 取得所有数据
             * @return {[type]} [description]
             */
            this.all = function(){
                return flat.get(datakey);
            };

            /**
             * 取得数量
             * @return {[type]} [description]
             */
            this.amount = this.count = function(){
                var data = that.all();
                return data ? data.length : 0;
            };

            /**
             * 根据参数取得或设置数据
             * @param  {[type]} data [description]
             * @return {[type]}      [description]
             */
            this.data = function(data) {
                if (data === undefined) {
                    return that.all();
                } else {
                    return that.put(data);
                }
            };

            /**
             * 清空数据列表
             * @return {[type]} [description]
             */
            this.clear = function(){
                return that.put([]);
            };

            /**
             * 在最前面插入数据
             * @param  {[type]} dat [description]
             * @return {[type]}      [description]
             */
            this.insert = this.preAppend = function(dat){
                flat.unshift(datakey, dat);
                return that;
            };

            /**
             * 在最后压入数据
             * @param  {[type]} dat [description]
             * @return {[type]}      [description]
             */
            this.push = this.append = function(dat) {
                flat.push(datakey, dat);
                return that;
            };

            /**
             * 弹掉最后一个
             * @return {[type]} [description]
             */
            this.pop = function() {
                flat.pop(datakey);
                return that;
            };

            /**
             * 根据index移除
             * @return {[type]} [description]
             */
            this.removeByIndex = function(index){
                flat.delete([datakey, index].join('.'));
                return that;
            }

            /**
             * 列表项操作集
             * @type {Object}
             */
            this.item = {
                /**
                 * [each description]
                 * @param  {Function} callback [description]
                 * @return {[type]}            [description]
                 */
                each: function(callback) {
                    angular.forEach(that.all(), callback);
                },

                /**
                 * 根据mix(id/dat) 获得最终dat
                 * @param  {[type]} mix  [description]
                 * @return {[type]}       [description]
                 */
                data: function(mix){
                    if (!angular.isObject(mix)) {
                        return that.all().find(function(item){
                            return mix === item[that.idKey];
                        });
                    } else {
                        return mix;
                    }
                },

                /**
                 * 根据指定的ids, 从列表提取相应数据集
                 * @param  {[type]} ids [description]
                 * @return {[type]}     [description]
                 */
                mapById: function(ids) {
                    if (!angular.isArray(ids)) ids = [ids];
                    return that.all().filter(function(item){
                        return ids.indexOf(item[that.idKey]) > -1;
                    });
                },

                /**
                 * 根据mix(id/dat) 获得其index
                 * @param  {[type]} mix [description]
                 * @return {[type]}      [description]
                 */
                mapIndex: function(mix){
                    var _item = this.data(mix);
                    return that.all().findIndex(function(item){
                        return item[that.idKey] == _item[that.idKey];
                    });
                },

                /**
                 * 根据指定的indexs, 从列表提取相应数据集
                 * @param  {[type]} indexs [description]
                 * @return {[type]}        [description]
                 */
                mapByIndex: function(indexs) {
                    if (!angular.isArray(indexs)) indexs = [indexs];
                    return that.all().filter(function(item, index){
                        return indexs.indexOf(index) > -1;
                    });
                },

                /**
                 * 根据index取得列表项
                 * @param  {[type]} index [description]
                 * @return {[type]}       [description]
                 */
                getByIndex: function(index) {
                    return flat.get([datakey, index].join('.'));
                },

                /**
                 * 交换2条数据顺序
                 * @param  {[type]} item1 [description]
                 * @param  {[type]} item2 [description]
                 * @return {[type]}       [description]
                 */
                swap:function(item1, item2){
                    var data = that.all();
                    var item1 = angular.extend({}, this.data(item1));
                    var index1 = this.mapIndex(item1);
                    var item2 = angular.extend({}, this.data(item2));
                    var index2 = this.mapIndex(item2);
                    data[index1] = item2;
                    data[index2] = item1;
                    return data;
                },

                /**
                 * 更新列表项
                 * @param  {[type]} item  [description]
                 * @param  {[type]} data  [description]
                 * @param  {[type]} field [description]
                 * @return {[type]}       [description]
                 */
                update: function(item, data, field) {
                    item = this.data(item);
                    if (field) {
                        item[field] = data;
                    } else {
                        angular.extend(item, data);
                    }
                },

                /**
                 * 移除列表项
                 * @param  {[type]} item [description]
                 * @return {[type]}      [description]
                 */
                remove: function(item) {
                    var index = this.mapIndex(item);
                    that.removeByIndex(index);
                },

                /**
                 * make一个listitem实例
                 * @param  {[type]} $event [description]
                 * @return {[type]}        [description]
                 */
                make:function($event) {
                    var instance = new Listitem();
                    instance.setObjectByEvent($event);
                    return instance;
                },

                /**
                 * 根据e
                 * @param  {[type]} $event [description]
                 * @return {[type]}        [description]
                 */
                getIndexByEvent: function($event) {
                    return this.make($event).getIndex();
                },

                /**
                 * 通过触发事源取得列表项
                 * @param  {[type]} $event [description]
                 * @return {[type]}        [description]
                 */
                getByEvent: function($event) {
                    return this.make($event).getObject();
                },


                /**
                 * 根据事件触发源更新列表项
                 * @param  {[type]} item  [description]
                 * @param  {[type]} data  [description]
                 * @param  {[type]} field [description]
                 * @return {[type]}       [description]
                 */
                updateByEvent: function($event, data) {
                    var index = this.getIndexByEvent($event);
                    var item = this.getByIndex(index);
                    this.update(item, data);
                },

                /**
                 * 根据事件来源选中相应列表项
                 * @param  {[type]} $event [description]
                 * @return {[type]}        [description]
                 */
                checkByEvent: function($event) {
                    return this.make($event).checked();
                },

                /**
                 * 根据事件来源取消选中相应列表项
                 * @param  {[type]} $event [description]
                 * @return {[type]}        [description]
                 */
                uncheckByEvent: function($event) {
                    return this.make($event).unchecked();
                },

                /**
                 * 根据事件来源激活相应列表项
                 * [activeByEvent description]
                 * @param  {[type]} $event        [description]
                 * @param  {[type]} actived_class [description]
                 * @return {[type]}               [description]
                 */
                activeByEvent: function($event, actived_class) {
                    return this.make($event).actived(actived_class);
                },

                /**
                 * 根据事件来源取消激活相应列表项
                 * @param  {[type]} $event        [description]
                 * @param  {[type]} actived_class [description]
                 * @return {[type]}               [description]
                 */
                unactiveByEvent: function($event, actived_class) {
                    return this.make($event).unactived(actived_class);
                }
            };
        };

        return {
            make:function(scope, datakey){
                return new list(scope, datakey);
            },
            _class:_class
        };
    }]);
;angular.module('Landers.angular')
    .factory('Location', ['$location', function($location){
        return {
            param:function(index, noNull){
                var url = $location.url().split('/');
                if (index == 'last') index = url.length - 1;
                var ret = url[index];
                if (!ret && noNull !== false) {
                    ret = url[index-1];
                }
                return ret;
            },
            params:function(keys){
                var url = $location.url().split('/');
                var ret = {};
                angular.forEach(keys, function(item, i){
                    ret[item] = url[i+2];
                });
                return ret;
            }
        };
    }]);
;angular.module('Landers.angular')
    .factory('Runtime', ['Flat', function (Flat) {
        var cache = {};
        function runtime (scope, inhert){
            var prekey = '$runtime';
            var watchers = {};

            if ( !cache[scope.$id] ) {
                if (inhert) {
                    scope.$runtime = scope.$runtime || {};
                } else {
                    scope.$runtime = {};
                }
                cache[scope.$id] = true;
            }
            this.__proto__ = Flat.make(scope[prekey], scope);
            this.watch = function(keys, callback){
                if (angular.isArray(keys)) {
                    var _watchers = [];
                    keys.map(function(key){
                        key = prekey + '.' + key;
                        watchers[key] = scope.$watch(key, callback);
                        _watchers[key] = watchers[key];
                    });
                    return _watchers;
                } else {
                    key = prekey + '.' + keys;
                    watchers[key] = scope.$watch(key, callback);
                    return watchers[key];
                }
            };
            this.destroyWatchers = function(keys){
                var that = this;
                if (!angular.isArray(keys)) keys = [keys];
                keys.map(function(key){
                    key = prekey + '.' + key;
                    console.log(key, typeof watchers[key], watchers[key]);
                    if (angular.isFunction(watchers[key])) {
                        watchers[key]();
                    }
                });
            }
        }
        return {
            make:function(scope, inhert){
                return new runtime(scope, inhert);
            }
        };
    }]);
;angular.module('Landers.angular')
    .factory('TabIndex', ['$state', 'Location', '$stateParams', function ($state, Location, $stateParams){
        function TabIndex(scope) {
            this.show = function(callbacks){
                var that = this;
                var tab_index = $stateParams.tabIndex;
                if ( !tab_index ) {
                    var stateData = $state.$current.data || {};
                    tab_index = typeof stateData.tabIndex !== 'undefined' ?
                                stateData.tabIndex : Location.param(2);
                }
                scope.tabIndex = tab_index;
                var _callback = callbacks[tab_index] || callbacks['default'];
                if (_callback) _callback(tab_index);
                else throw('callbacks[' + tab_index + ']未定义');
            };

        }
        return {
            make:function(_scope){return new TabIndex(_scope);},
            get:function(){return Location.param(2);}
        };
    }]);
;angular.module('Landers.angular')
    .factory('TreeJson', [function(){
        var TreeJson = function(data){
            this.data = data;
            this.keys = {
                'id': 'key',
                'childs': 'subs'
            };

            var that = this;
            function findIndex(array, key){
                var id = that.keys.id;

                var index = array.findIndex(function(item, index, arr){
                    return item[id] == key
                });
                if ( isNaN(index) ) {
                    throw '未找到标识为' + key + '的分支';
                }
                return array[index];
            };
            this.setKeys = function(id, childs){
                if ( id ) that.keys.id = id;
                if ( childs ) that.keys.childs = childs;
            };

            this.getByKeys = function(keys, key){
                var data = that.data;
                var childs = that.keys.childs;
                var nodes = [];

                var keys_left = keys.slice(0, keys.length-1);
                var last_key = keys[keys.length-1];
                if ( keys_left.length ) {
                    keys_left.map(function(_key){
                        var json = findIndex(data, _key);
                        var json_clone = angular.extend({}, json);
                        delete json_clone[childs];
                        nodes.push( key ? json_clone[key] : json_clone);


                        data = json[childs];
                    });
                }

                var dat = findIndex(data, last_key);
                nodes.push( key ? dat[key] : dat);

                return nodes;

            };

            this.getByIndexs = function(indexs){
                var obj = that.data;
                var id = that.keys.id;
                var childs = that.keys.childs;

                indexs.map(function(index){
                    obj = obj[index][childs];
                });

                return obj;
            };

            this.find = function(keys, key){
                var nodes = that.get(keys);
                var last_node = nodes[nodes.length-1];
                return key ? last_node[key] : last_node;
            };
        };
        return {
            make:function(data){return new TreeJson(data);},
        };
    }]);
;angular.module('Landers.angular')
    .filter('datetime', ['$filter', function($filter){
        return function(input, format) {
            if ( !input ) return;
            var str = typeof(input) == 'number' ? $filter("date")(input * 1000, 'yyyy-MM-dd HH:mm:ss') : input;
            var a = str.split(' ');
            switch (format) {
                case 'date' : return a[0];
                case 'time' : return a[1];
                default : return str;
            }
        };
    }]);
;angular.module('Landers.angular')
    .filter('nl2br', function(){
        return function(str) {
            var s1 = '\n'; s2 = '<br/>';
            s1 = s1.replace(/\{/,'\\{').replace(/\}/,'\\}');
            var re = new RegExp(s1,"gi");
            return str.replace(re,s2);
        };
    });
;angular.module('Landers.angular')
    .filter('nl2li', function(){
        return function(str) {
            return (str || '').split('\n').map(function(line){
                return '<li>' + line + '</li>';
            }).join('');
        };
    });
;angular.module('Landers.angular')
    .filter('round', function() {
        return function(num, n){return Math.round(num*Math.pow(10,n || 0))/Math.pow(10,n || 0); };
    });
;angular.module('Landers.angular')
    .filter('trusted', ['$sce', function ($sce) {
        return function(text) {
            return $sce.trustAsHtml(text);
        };
    }])
;angular.module('Landers.angular')
    .filter('join', function(){
        return function(arr, link) {
            if (!angular.isArray(arr)) return arr;
            var ret = arr.join(link);
            alert(ret);
            return ret;
        };
    });
;angular.module('Landers.angular')
    .filter('replace', function(){
        return function(str, str1, str2) {
            if (!str) return str;
            var reg = new RegExp( str1 , 'g' );
            if (typeof str != 'string') {
                str = angular.toJson(str);
            }
            return str.replace( reg , str2 );
        };
    });
;angular.module('Landers.angular')
    .provider('Ajax', function() {
        var provider = this;
        this.hosturl = '';
        this.$get = ['$http', '$rootScope', '$q', function($http, $rootScope, $q){
            var csrf_token;
            function Ajax() {
                var _debug, _loading;
                var _hosturl = provider.hosturl;
                var cancellers = [];
                var prevents = {
                    error : false,
                    error_once : false
                };
                var build_url = function(url, params) {
                    if (params) {
                        url = Landers.url.qs.set(url, params);
                    }
                    return _hosturl + url;
                };
                this.setHosturl = function(hosturl){
                    _hosturl = hosturl;
                    return this;
                }
                this.debug = function(is){
                    _debug = is !== false && is !== 0;
                    return this;
                };
                this.loading = function(is){
                    _loading = is === true || is === undefined;
                    return this;
                };
                this._get = function(method, url, params){
                    var that = this;
                    url = build_url(url, params);

                    var ret, config = {ignoreLoadingBar: !_loading};
                    var canceler = $q.defer();
                    cancellers.push(canceler);
                    // 添加timeout设置为promise对象，才可以取消未完成的request请求
                    config.timeout = canceler.promise;
                    switch (method.toLowerCase()) {
                        case 'get' :
                            ret = $http.get(url, config); break;
                        // case 'delete':
                        //     ret = $http.delete(url, config); break;
                    }
                    ret.success(that.success).error(that.error)
                       .finally(function(){cancellers.pop();});
                    _loading = _debug = false;
                    return ret;
                };
                this.get = function(url, params) {
                    return this._get('get', url, params);
                };
                this.delete = function(url, data){
                    return this._post('delete', url, data);
                };
                this._post = function(method, url, xvar, params){
                    var that = this;
                    url = build_url(url, params);
                    // var data = angular.isString(xvar) || angular.isElement(xvar) ? Form.data(xvar) : xvar;
                    var data = xvar || {};

                    if (csrf_token) data._token = csrf_token;

                    var ret, config = {ignoreLoadingBar: !_loading}; //, withCredentials:true
                    var canceler = $q.defer();
                    cancellers.push(canceler);
                    // 添加timeout设置为promise对象，才可以取消未完成的request请求
                    config.timeout = canceler.promise;
                    switch (method.toLowerCase()) {
                        case 'post' :
                            ret = $http.post(url, data, config); break;
                        case 'put' :
                            data['_method'] = 'put';
                            ret = $http.put(url, data, config); break;
                        case 'delete':
                            config.data = data;
                            config.headers = {
                                'Content-type': 'application/json;charset=utf-8'
                            };
                            ret = $http.delete(url, config); break;
                    }
                    ret.success(that.success).error(that.error)
                       .finally(function(){cancellers.pop();});

                    _loading = _debug = false;
                    return ret;

                };
                this.post = function(url, xvar, params){
                    return this._post('post', url, xvar, params);
                };
                this.put = function(url, data, params){
                    return this._post('put', url, data, params);
                };
                this.success = function(rspn, code, header){
                    csrf_token = rspn.csrf_token;
                    if (rspn.code == 1 && rspn.message) {
                        Landers.msgbox(rspn);
                    }
                };
                this.error = function(rspn) {
                    if (angular.isObject(rspn)) {
                        csrf_token = rspn.csrf_token || csrf_token;
                    } else {
                        console.log('无响应404');
                        rspn = {
                            success : false,
                            message : '' //'远程无响应或OPTION方法',
                        };
                    }
                };
                this.cancelAll = function(i){
                    if ( cancellers.length ) {
                        var canceller;
                        while (canceller = cancellers.pop()) {
                            canceller.resolve("user cancelled");
                        }
                    }
                };
            }
            return {
                make:function(){
                    return new Ajax();
                },
                csrf_token:function(){
                    return csrf_token;
                }
            };
        }];
    });
;angular.module('Landers.angular')
    .provider('Dialog', function(){
        var provider = this;
        this.skin = 'ulan';
        this.defaults = {
            effect : 'random'
        };
        this.$get = ['$compile', '$timeout', function($compile, $timeout) {

            //先异步加载css文件，以便Landers.dialog.show为非异步
            Landers.dialog.loadmod(provider.skin);
            return {
                confirm:function(scope, opts, callbacks){
                    callbacks = callbacks || {};
                    callbacks.yesFn = callbacks.yesFn || function () {};
                    callbacks.noFn = callbacks.noFn || function () {};

                    angular.extend(opts, {
                        top : '30%',
                        yes: {text:'　确定　', fn:callbacks.yesFn },
                        no: {text:'　取消　', fn:callbacks.noFn },
                        blur: false,
                        angular : {compile:$compile, scope:scope, timeout:$timeout}
                    });
                    return Landers.dialog.show(opts);
                },
                show:function(scope, opts){
                    var extOpts = opts.compile === false ? {} : {
                        angular : {compile:$compile, scope:scope, timeout:$timeout},
                    };

                    opts = angular.extend({}, provider.defaults, extOpts, opts);
                    return Landers.dialog.show(opts, provider.skin);
                }
            };
        }];
    });
;angular.module('Landers.angular')
    .factory('RouteMenu', ['$state', function($state){
            function RouteMenu( selector ){
                var that = this;

                this.getMenu = function( callback ){
                    var $menu = $(selector);

                    if ( !$menu.length || !$menu.children().length) {
                        var self = arguments.callee;
                        setTimeout(function(){
                            self(callback);
                        }, 300);
                    } else {
                        callback && callback( $menu );
                    }

                    return that;
                };

                this.findForActive = function(callback){
                    var current_state = $state.current.name;
                    var active_states = $state.current.activeStates;
                    if ( !active_states || !active_states.length ) {
                        active_states = [ current_state ];
                    }

                    that.getMenu( function( $menu ){
                        var $current_states = $(), $other_states = $();
                        angular.forEach(active_states, function(item){
                            if ( item === '$$SELF' ) {
                                item = current_state;
                            }
                            var $find = $menu.find("[ui-sref^='" + item + "(']");
                            if ( current_state === item ) {
                                $current_states = $current_states.add($find);
                            } else {
                                $other_states = $other_states.add($find);
                            }
                        });

                        var $all_states = $current_states.add($other_states);

                        callback && callback(
                            $all_states,
                            $current_states,
                            $other_states
                        );
                    });

                    return that;
                }
            }

            return {
                make: function(selector){
                    return new RouteMenu(selector);
                }
            }
        }]);
;angular.module('Landers.angular')
    .directive('listStatus', ['$templateRequest', '$compile', 'Flat', 'Helpers', '$timeout', function($templateRequest, $compile, Flat, Helpers, $timeout){
        return {
            restrict : 'A',
            link : function($scope, $ele, $attrs){
                $ele.addClass('list-status');
                var myScope = Helpers.seekScope($scope, $attrs['useScope']);

                var modelKey = $attrs['modelKey'], html = '';
                var datakey1 = [modelKey, '$list', 'status'].join('.');
                var datakey2 = [modelKey, '$list', 'data'].join('.');
                var icon1 = '<i class="glyphicon glyphicon-info-sign"></i> ';
                var icon2 = '<i class="glyphicon glyphicon-remove-sign"></i>';
                myScope.$watch(datakey1, function(newValue){
                    switch(newValue) {
                        case 'LOADING':
                            var html = '<div class="alert alert-default pv20"><i class="glyphicon glyphicon-cog glyphicon-spin fa fa-cog fa-spin"></i> loading... </div>';
                            $ele.html($compile(html)(myScope)).hide().fadeIn(500);
                            break;
                        case 'COMPLETED':
                            $ele.empty();
                            break;
                    }

                });
                myScope.$watch(datakey2, function(newValue){
                    if (newValue) {
                        if ( !newValue.length ) {
                            var datakey2 = [modelKey, '$errors.$list'].join('.');
                            var error_message = Flat.get(myScope, datakey2);
                            if ( !error_message ) {
                                datakey3 = [modelKey, '$search'].join('.');
                                var htmlTpl = '<div class="alert alert-warning pv20">%s</div>';
                                var datakey4 = [modelKey, '$name'].join('.');
                                var appname = Flat.get(myScope, datakey4);

                                var searchData = Flat.get(myScope, datakey3);
                                for (var k in searchData) {
                                    if ( searchData[k] === '' ) delete searchData[k];
                                }

                                var html;
                                if ( searchData && Landers.utils.json.count(searchData) ) {
                                    html = icon1 + ' 没有检索到符合条件的' + appname + '。您可以尝试其他搜索条件或<a class="highlight hand" ng-click="' + modelKey + '.$event.resetSearch()">重置搜索</a> ';
                                    html = Landers.utils.str.sprintf(htmlTpl, html);
                                } else {
                                    var partial = $attrs['partialNone'];
                                    var message = $attrs['messageNone'];
                                    if (partial) {
                                        var partial_url = myScope.$partial(partial);
                                        $templateRequest(partial_url).then(function(html){
                                            html = Landers.utils.str.sprintf(htmlTpl, html);
                                            $ele.fadeOut(300, function(){
                                                $timeout(function(){//必须加延时，否则可能html未解析
                                                    $ele.html($compile(html)(myScope)).fadeIn(300);
                                                }, 100);
                                            });
                                        });
                                        return;

                                    } else if (message) {
                                        html = message;
                                    } else {
                                        html = '暂无' + appname + '。';
                                    }
                                    html = Landers.utils.str.sprintf(htmlTpl, icon1 + html);
                                }
                            } else {
                                html = '<div class="alert alert-danger pv20">' + icon2 + ' ' + error_message + ' </div>';
                            }
                            $ele.fadeOut(300, function(){
                                $ele.html($compile(html)(myScope)).fadeIn(300);
                            });
                        } else {
                            $ele.empty().stop(true).hide();
                        }
                    }
                }, true);
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('uiAlertError', ['Flat', function(Flat){
        return {
            restrict : 'A',
            link : function($scope, $ele, $attrs){
                var datKey = $attrs['uiAlertError'];
                $ele.addClass('alert alert-danger').hide();
                $scope.$watch(datKey, function(newValue){
                    if (newValue) {
                        var html = '<i class="fa fa-minus-circle"></i> ' + newValue;
                        $ele.html(html).show();
                    } else {
                        $ele.hide();
                    }
                });
                var $form = $ele.closest('form');
                $form.find('input, select, textarea').change(function(){
                    var ele = $(this);
                    if (!ele.val()) return;
                    var ngmodel = ele.attr('ng-model');
                    var arr = ngmodel.split('.');
                    arr.splice(1, 0, '$errors');
                    var datkey = arr.join('.');
                    Flat.set($scope, datkey, '');
                    angular_scope_apply($scope);
                });
            }
        };
    }])
;angular.module('Landers.angular')
    .directive('checklistModel', ['$parse', '$compile', function($parse, $compile) {
        // contains
        function contains(arr, item, comparator) {
            if (angular.isArray(arr)) {
                for (var i = arr.length; i--;) {
                    if (comparator(arr[i], item)) {
                        return true;
                    }
                }
            }
            return false;
        }

        // add
        function add(arr, item, comparator) {
            arr = angular.isArray(arr) ? arr : [];
            if (!contains(arr, item, comparator)) {
                arr.push(item);
            }
            return arr;
        }

        // remove
        function remove(arr, item, comparator) {
            if (angular.isArray(arr)) {
                for (var i = arr.length; i--;) {
                    if (comparator(arr[i], item)) {
                        arr.splice(i, 1);
                        break;
                    }
                }
            }
            return arr;
        }

        // http://stackoverflow.com/a/19228302/1458162
        function postLinkFn(scope, elem, attrs) {
            // exclude recursion, but still keep the model
            var checklistModel = attrs.checklistModel;
            attrs.$set("checklistModel", null);
            // compile with `ng-model` pointing to `checked`
            $compile(elem)(scope);
            attrs.$set("checklistModel", checklistModel);

            // getter for original model
            var checklistModelGetter = $parse(checklistModel);
            var checklistChange = $parse(attrs.checklistChange);
            var checklistBeforeChange = $parse(attrs.checklistBeforeChange);
            var ngModelGetter = $parse(attrs.ngModel);



            var comparator = angular.equals;

            if (attrs.hasOwnProperty('checklistComparator')) {
                if (attrs.checklistComparator[0] == '.') {
                    var comparatorExpression = attrs.checklistComparator.substring(1);
                    comparator = function(a, b) {
                        return a[comparatorExpression] === b[comparatorExpression];
                    };

                } else {
                    comparator = $parse(attrs.checklistComparator)(scope.$parent);
                }
            }

            // watch UI checked change
            scope.$watch(attrs.ngModel, function(newValue, oldValue) {
                if (newValue === oldValue) {
                    return;
                }

                if (checklistBeforeChange && (checklistBeforeChange(scope) === false)) {
                    ngModelGetter.assign(scope, contains(checklistModelGetter(scope.$parent), getChecklistValue(), comparator));
                    return;
                }

                setValueInChecklistModel(getChecklistValue(), newValue);

                if (checklistChange) {
                    checklistChange(scope);
                }
            });

            // watches for value change of checklistValue (Credit to @blingerson)
            scope.$watch(getChecklistValue, function(newValue, oldValue) {
                if (newValue != oldValue && angular.isDefined(oldValue) && scope[attrs.ngModel] === true) {
                    var current = checklistModelGetter(scope.$parent);
                    checklistModelGetter.assign(scope.$parent, remove(current, oldValue, comparator));
                    checklistModelGetter.assign(scope.$parent, add(current, newValue, comparator));
                }
            });

            function getChecklistValue() {
                return attrs.checklistValue ? $parse(attrs.checklistValue)(scope.$parent) : attrs.value;
            }

            function setValueInChecklistModel(value, checked) {
                var current = checklistModelGetter(scope.$parent);
                if (angular.isFunction(checklistModelGetter.assign)) {
                    if (checked === true) {
                        checklistModelGetter.assign(scope.$parent, add(current, value, comparator));
                    } else {
                        checklistModelGetter.assign(scope.$parent, remove(current, value, comparator));
                    }
                }

            }

            // declare one function to be used for both $watch functions
            function setChecked(newArr, oldArr) {
                if (checklistBeforeChange && (checklistBeforeChange(scope) === false)) {
                    setValueInChecklistModel(getChecklistValue(), ngModelGetter(scope));
                    return;
                }
                ngModelGetter.assign(scope, contains(newArr, getChecklistValue(), comparator));
            }

            // watch original model change
            // use the faster $watchCollection method if it's available
            if (angular.isFunction(scope.$parent.$watchCollection)) {
                scope.$parent.$watchCollection(checklistModel, setChecked);
            } else {
                scope.$parent.$watch(checklistModel, setChecked, true);
            }
        }

        return {
            restrict: 'A',
            priority: 1000,
            terminal: true,
            scope: true,
            compile: function(tElement, tAttrs) {

                if (!tAttrs.checklistValue && !tAttrs.value) {
                    throw 'You should provide `value` or `checklist-value`.';
                }

                // by default ngModel is 'checked', so we set it if not specified
                if (!tAttrs.ngModel) {
                    // local scope var storing individual checkbox model
                    tAttrs.$set("ngModel", "checked");
                }

                return postLinkFn;
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('bool', ['Flat', function(Flat){
        return {
            restrict : 'AE',
            require : 'ngModel',
            template : function($ele, attrs){
                var _class = attrs['class'] || '';
                return '<span class="label label-yes ' + _class + '">是</span><span class="label label-no ' + _class + '">否</span>'
            },
            link : function($scope, $ele, $attrs){
                var ngModel = $attrs['ngModel'];
                var $childs = $ele.children();
                $scope.$watch(ngModel, function(new_value){
                    var bool = !!~~new_value;
                    if (bool) {
                        $childs.eq(0).show();
                        $childs.eq(1).hide();
                    } else {
                        $childs.eq(0).hide();
                        $childs.eq(1).show();
                    }
                });

            }
        }
    }]);
;angular.module('Landers.angular')
    .directive('bootstrapTooltip', [function(){
        return {
            restrict : 'C',
            link : function($scope, $ele, $attrs){
                (function(){
                    var self = arguments.callee;
                    var title = $ele.attr('data-title');
                    if ( !title || title.match(/\{\{(.*)\}\}/) ) {
                        setTimeout(self, 300);
                    } else {
                        $ele.tooltip();
                    }
                })();
            }
        }
    }]);
;angular.module('Landers.angular')
    .factory('DatatimePickerHelper', ['Flat', '$timeout', function(Flat, $timeout){
        function DatatimePickerHelper(scope, ngModel, $begin, $end) {
            var ngModelBegin = $begin.attr('ng-model');
            var ngModelEnd = $end.attr('ng-model');
            this.set = function(begin, end){
                $timeout(function(){
                    begin = begin || '';
                    end = end || '';
                    if (ngModelBegin) {
                        Flat.set(scope, ngModelBegin, begin);
                    } else {
                        $begin.val(begin);
                    }
                    if (ngModelEnd) {
                        Flat.set(scope, ngModelEnd, end);
                    } else {
                        $end.val(end);
                    }
                    if (ngModel) {
                        Flat.set(scope, ngModel, begin || end ? [begin, end] : undefined);
                    }
                    if (ngModelBegin || ngModelEnd || ngModel) scope.$apply();
                }, 100);
            };
        }

        return {
            make : function(scope, ngModel, $begin, $end){
                return new DatatimePickerHelper(scope, ngModel, $begin, $end);
            },
            locale : {
                'zh-cn' : {
                    today : '定位今天',
                    clear : '清空',
                    close : '关闭',
                    selectMonth: '选择月份',
                    prevMonth: '上月',
                    nextMonth: '下月',
                    selectYear: '选择年份',
                    prevYear: '上年',
                    nextYear: '下年',
                    selectDecade: 'Select Decade',
                    prevDecade: 'Previous Decade',
                    nextDecade: 'Next Decade',
                    prevCentury: 'Previous Century',
                    nextCentury: 'Next Century',
                    pickHour: '选取小时',
                    incrementHour: '增加小时',
                    decrementHour: '减少小时',
                    pickMinute: '选取分钟',
                    incrementMinute: '增加分钟',
                    decrementMinute: '减少分钟',
                    pickSecond: '选取秒',
                    incrementSecond: '增加秒',
                    decrementSecond: '减少秒',
                    togglePeriod: 'Toggle Period',
                    selectTime: '选择时间'
                }
            }
        };
    }])
    // .directive('bootstrapDatetimeRangePicker', ['Flat', '$timeout', 'DatatimePickerHelper', function(Flat, $timeout, DatatimePickerHelper){
    //     return {
    //         restrict : 'A',
    //         link : function($scope, $ele, $attrs){
    //             var ngModel = $attrs['ngModel'];
    //             var $begin = $ele.siblings('.datetime-begin');
    //             var $end = $ele.siblings('.datetime-end');
    //             var helper = DatatimePickerHelper.make($scope, ngModel, $begin, $end);

    //             var format = $ele.data('format') || 'YYYY-MM-DD hh:mm A';

    //             var defaults = {
    //                 showISOWeekNumbers: true,
    //                 timePicker24Hour: true,
    //                 alwaysShowCalendars: true,
    //                 opens: 'left',
    //                 timePicker: true,
    //                 timePickerIncrement: 1,
    //                 locale: {
    //                     format: format,
    //                     cancelLabel: 'Clear'
    //                 }
    //             };
    //             var config = $ele.data();
    //             delete config.$ngModelController;
    //             config = angular.extend({}, defaults, config);

    //             $ele.daterangepicker(config, function(begin, end, label) {
    //                 begin = begin.format(config.format);
    //                 end = end.format(config.format);
    //                 helper.set(begin, end);
    //             });

    //             $ele.on('cancel.daterangepicker', function(ev, picker) {
    //                 $ele.val('');
    //             });
    //             $ele.val('');
    //         }
    //     };
    // }])
    .directive('bootstrapDatetimePicker', ['Flat', '$timeout', 'DatatimePickerHelper', function(Flat, $timeout, DatatimePickerHelper){
        return {
            restrict : 'A',
            link : function($scope, $ele, $attrs){
                var ngModel = $attrs['ngModel'];
                $ele.addClass('datetime-picker');
                var $begin = $ele.children('.datetime-begin');
                var $end = $ele.children('.datetime-end');
                if (!$begin.length && !$end.length) {
                    $begin = $end = $ele.find('input');
                }

                var helper = DatatimePickerHelper.make($scope, ngModel, $begin, $end);

                var defaults = {
                    format : 'YYYY-MM-DD HH:mm:ss',
                    showClose:true,
                    showClear:true,
                    sideBySide:false,
                    showTodayButton: true,
                    locale: 'zh-cn',
                    tooltips:DatatimePickerHelper.locale['zh-cn']
                };
                var config = $ele.data();
                delete config.$ngModelController;
                delete config.minDate;
                delete config.maxDate;
                config = angular.extend({}, defaults, config);

                $begin.add($end).datetimepicker(config);

                var values = {begin:'', end:''};
                $begin.on('dp.change', function(e){
                    if ( e.date) {
                        values.begin = e.date.format(config.format);
                    } else {
                        values.begin = '';
                    }
                    helper.set(values.begin, values.end);
                });
                $end.on('dp.change', function(e){
                    if ( e.date ) {
                        values.end = e.date.format(config.format);
                    } else {
                        values.end = '';
                    }
                    helper.set(values.begin, values.end);
                });
                $begin.add($end).on('dp.show', function(){
                    var $wrapper = $(this).next();
                    var dtp = $(this).data('DateTimePicker');
                    var minDate = $(this).data('minDate');
                    if (minDate) dtp.minDate(minDate);
                    var maxDate = $(this).data('maxDate');
                    if (maxDate) dtp.maxDate(maxDate);

                    setTimeout(function(){
                        $wrapper.find('.timepicker-picker a.btn').removeClass('btn');
                    }, 200);
                });
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('disabledWhen', ['$compile', 'Runtime', 'Helpers', function($compile, Runtime, Helpers){
        return {
            restrict : 'A',
            scope: {
                when: '@disabledWhen'
            },
            link : function($scope, $ele, $attrs) {
                var myScope = Helpers.seekScope($scope, $attrs['useScope']);

                myScope.$watch('when', function(new_value){
                    switch (new_value) {
                        case 'true' : new_value = true; break;
                        case 'false' : new_value = false; break;
                    }

                    if (new_value) {
                        $ele.addClass('disabled').prop('disabled', true);
                    } else {
                        $ele.removeClass('disabled').prop('disabled', false);
                    }
                });
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('icheck', ['$timeout', '$parse', function ($timeout, $parse) {
        return {
            require: 'ngModel',
            link: function($scope, element, $attrs, ngModel) {

                return $timeout(function() {
                    var value;
                    value = $attrs['value'];

                    $scope.$watch($attrs['ngModel'], function(newValue){
                        $(element).icheck('update');
                    });

                    Landers.loader.jsm('jquery.icheck,true', function(){
                        return $(element).icheck({
                            checkboxClass: 'icheckbox_square-blue',
                            radioClass: 'iradio_square-blue'

                        }).on('ifChanged', function(event) {
                            if ($(element).attr('type') === 'checkbox' && $attrs['ngModel']) {
                                $scope.$apply(function() {
                                    return ngModel.$setViewValue(event.target.checked);
                                });
                            }
                            if ($(element).attr('type') === 'radio' && $attrs['ngModel']) {
                                return $scope.$apply(function() {
                                    return ngModel.$setViewValue(value);
                                });
                            }
                        });
                    });

                });
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('pagebar', ['$compile', 'Flat', function($compile, Flat){
        return {
            restrict : 'A',
            link : function($scope, $ele, $attrs){
                var modelKey = $attrs['modelKey'];
                var pageinfo = Landers.utils.to.bool($attrs['pageinfo']);
                $scope.$watch (modelKey + '.$list', function($list){
                    if ( !$list ) return;

                    var current, total, size, last;
                    if ( $list.page ) {
                        current = '%M%.$list.page.current';
                        total = '%M%.$list.page.total';
                        size = '%M%.$list.page.size';
                        last = '%M%.$list.page.last';
                    } else {
                        current = '%M%.$list.current_page';
                        total = '%M%.$list.total';
                        size = '%M%.$list.per_page';
                        last = '%M%.$list.last_page';
                    }
                    var html = '', paging_action = $attrs['pagingAction'] || '%M%.$event.showPage(page)';
                    html += '<div ng-if="' + last + ' > 0 && ' + total + ' > 0 && ' + size + ' > 0">';
                    if (pageinfo) {
                        html += '<div class="fl page-info mr5">';
                        html += '   <ul class="hlist pageinfo">';
                        html += '       <li>页次：{{' + current + '}}/{{' + last + '}}</li>';
                        html += '       <li class="ml15">每页：{{' + size + '}}</li>';
                        html += '       <li class="ml15">总数：{{' + total + '}}</li>';
                        html += '       <li class="ml15" ng-show="%M%.$items.selected.length">选中: {{%M%.$items.selected.length}}</li>';
                        html += '   </ul>';
                        html += '</div>';
                        html += '<div class="fr ml5">';
                        html += '   <div class="page-num paging" paging page="' + current + '" page-size="' + size + '" total="' + total + '" paging-action="' + paging_action + '" transclude="true"></div>';
                        html += '</div>';
                    } else {
                        html += '<div paging class="page-num paging text-center" page="' + current + '" page-size="' + size + '" total="' + total + '" paging-action="' + paging_action + '" transclude="true"></div>';
                    }
                    html += '</div>';
                    html = html.replace(/%M%/g, modelKey);
                    $ele.html($compile(html)($scope));
                });
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('protip', [function(){
        if ( !$.protip ) Landers.loader.jsm('jquery.protip,true');
        var is_protip = false;
        return {
            restrict : 'C',
            link : function($scope, $ele, $attrs){
                var self = arguments.callee;
                if ( !$ele.attr('data-pt-size') ) {
                    $ele.attr('data-pt-size', 'small');
                }
                if ( !$ele.attr('data-pt-scheme') ) {
                    $ele.attr('data-pt-scheme', 'black');
                }

                if ($.protip && $ele.attr('data-pt-title')) {
                    $.protip();
                    $ele.on('protipshow', function(itemInstance){
                        var $me = $(itemInstance.target);
                        var t = $me.data('ptShow');
                        if (t !== undefined && t !== true) {
                            setTimeout(function(){
                                $me.protipHide();
                            }, 5);
                        }
                    });
                } else {
                    setTimeout(function(){
                        self($scope, $ele, $attrs);
                    }, 500);
                }
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('remedySelect', function() {
        return {
            restrict : 'A',
            link : function($scope, $element, $attrs){
                var allow_mouseout = true;
                var blur_click = function(){
                    allow_mouseout = false;
                    setTimeout(function(){allow_mouseout = true;}, 100);
                };
                $element.mouseover(function(){
                    $(this).addClass('onhover');
                })
                .mouseout(function(){
                    if (allow_mouseout) {
                        $(this).removeClass('onhover onfocus');
                    }
                })
                .blur(function(){
                    blur_click();
                    $(this).removeClass('onhover onfocus');
                })
                .click(function(){
                    blur_click();
                    $(this).addClass('onfocus').removeClass('onhover');
                });
            }
        };
    });
;angular.module('Landers.angular')
    .directive('landersNameValueInputer', ['$timeout', 'Dialog', 'Helpers', function ($timeout, Dialog, Helpers) {
        var dialog_html = [
            '<form class="form-horizontal">',
            '   <table cellpadding="0" cellspacing="0" border="0" class="stereo form-table">',
            '   <tr>',
            '       <td><label class="control-label">名称</label></td>',
            '       <td><div class="control-input">',
            '           <input ng-model="form.name" maxlength="30" value="" type="text" class="form-control" ng-if="!options.names"/>',
            '           <select ng-model="form.name" class="form-control" ng-if="options.names">',
            '               <option ng-repeat="item in options.names" value="{{item}}">{{item}}</option>',
            '           </select>',
            '       </div></td>',
            '   </tr>',
            '   <tr>',
            '       <td><label class="control-label">数据</label></td>',
            '       <td><div class="control-input"><textarea ng-model="form.value" value="" type="text" class="form-control"></textarea></div></td>',
            '   </tr>',
            '   </table>',
            '</form>'
        ].join('\n');

        return {
            restrict: 'EA',
            scope: {
                nviModel: '='
            },
            template: [
                '<div class="relative">',
                '    <a ng-click="edit($event)" class="btn btn-xs btn-primary absolute right-top"><i class="glyphicon glyphicon-plus"></i></a>',
                '</div>',
                '<ul class="hlist list">',
                '    <li ng-repeat="item in nviModel track by $index" class="mr20 list-item">',
                '        <a ng-click="edit($event, $index)" class="">{{item.name}}：{{item.value}}</a>',
                '        <a ng-click="remove($event, $index)" class="btn btn-warning btn-xs ml5"><i class="glyphicon glyphicon-minus"></i></a>',
                '    </li>',
                '</ul>'
            ].join('\n'),
            link: function ($scope, $element, $attrs) {
                angular.extend($scope, {
                    form: {},
                    options: Helpers.attrItemToJson($attrs, 'nviOptions'),
                    edit: function($event, item_index) {
                        $timeout(function() {
                            Dialog.show($scope, {
                                title: $attrs['nviName'],
                                content: dialog_html,
                                width: 350,
                                lock: true,
                                fixed: true,
                                effect: 'vmove1',
                                onclose: function(){
                                    // 清除编辑数据
                                    $scope.form = {};
                                    angular_scope_apply($scope);
                                },
                                onload: function () {
                                    $scope.nviModel = $scope.nviModel || [];
                                    if (item_index !== undefined) {
                                        $scope.form = angular.copy($scope.nviModel[item_index]);
                                    }
                                },
                                yes: function () {
                                    var form_data = angular.copy($scope.form);
                                    var check_json_name_exists = function (json, json_index) {
                                        var index = $scope.nviModel.findIndex(function (item, item_index) {
                                            return item.name == json.name && json_index != item_index;
                                        });
                                        return index > -1;
                                    };

                                    // 检查数据完整性
                                    if (!form_data.name || !form_data.name.length ||
                                        !form_data.value || !form_data.value.length
                                    ) {
                                        alert('数据输入不完整');
                                        return false;
                                    }

                                    // 检查数据重复性
                                    if (check_json_name_exists(form_data, item_index)) {
                                        alert('【' + form_data.name + '】已经存在');
                                        return false;
                                    }

                                    if (!item_index) {
                                        //添加
                                        $scope.nviModel.push(form_data);
                                    } else {
                                        //编辑
                                        $scope.nviModel[item_index] = form_data;
                                    }

                                    angular_scope_apply($scope);
                                },
                                no: true
                            });
                        }, 100);
                    },
                    remove: function ($event, item_index) {
                        $scope.nviModel.splice(item_index, 1);
                    }
                });

                $scope.$watch('nviModel', function (new_value) {
                    if (new_value===undefined) {
                        return;
                    } else {
                        //转换字符串成数组
                        $scope.nviModel = angular.fromJson(new_value);
                    }
                });
            }
        };
    }])
    .filter('landersParseNameValue', function(){
        return function(json, link) {
            link = link || '';

            var ret = [];
            for (var i=0; i<json.length; i++) {
                ret.push(json[i].name + ':' + json[i].value);
            }

            return ret.join(link);
        };
    })
;
;angular.module('Landers.angular')
    .directive('dropSelect', ['Flat', function(Flat) {
        return {
            restrict : 'A',
            link : function($scope, $ele, $attrs){
                var $button = $ele.find('.dropdown-toggle');
                var $list = $ele.find('.dropdown-menu');

                var init_event = $attrs['dropSelectInit'];
                init_event = Flat.get($scope, init_event);
                if (init_event) init_event();

                var change_event = $attrs['dropSelectChange'];
                change_event = Flat.get($scope, change_event);
                if (change_event) {
                    (function(){
                        var self = arguments.callee;
                        var $items = $list.children('li');
                        if ($items.length) {
                            $items.on('click', function(){
                                change_event($(this));
                            });
                        } else {
                            setTimeout(self, 300);
                        }
                    })();
                }
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('includeReplace', function () {
        return {
            require: 'ngInclude',
            restrict: 'A', /* optional */
            link: function (scope, el, attrs) {
                el.replaceWith(el.children());
            }
        };
    })
;angular.module('Landers.angular')
    .directive('mCustomerScroll', [function(){
        return {
            restrict : 'A',
            link : function($scope, $ele, $attrs) {
                setTimeout(function(){
                    $ele.mCustomScrollbar({
                        autoHideScrollbar:true,
                        scrollInertia:550,
                        theme:'minimal-dark',
                        preventDefault:true,
                        mouseWheel:true
                    });
                }, 300);
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('iscroll', [function(){
        return {
            restrict : 'A',
            link : function($scope, $ele, $attrs) {
                $ele.css({overflow:'hidden'});

                var options = $attrs['iscrollOption'] || '{}';
                options = angular.fromJson(options) || {};
                options = angular.extend({
                    scrollbars: true,
                    mouseWheel: true,
                    click: true
                }, options);

                var styles = $attrs['iscrollStyle'];
                var $wraper = $ele.children(':eq(0)');

                (function(){
                    var self = arguments.callee;
                    if ($wraper.children().length) {
                        new IScroll($ele.get(0), options);
                        setTimeout(function(){
                            $ele.find('.iScrollIndicator').css({
                                border: 'none',
                                width: '4px',
                                right: '0px'
                            }).get(0).style.cssText += styles;
                        }, 100);
                    } else {
                        setTimeout(self, 300);
                    }
                })();
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('loadingIcon', ['Flat', function(Flat){
        return {
            restrict : 'E',
            template : '<i class="glyphicon glyphicon-cog glyphicon-spin fa fa-cog fa-spin"></i>',
            link : function($scope, $ele, $attrs){

            }
        }
    }]);
;angular.module('Landers.angular')
    .directive('inputRadioOrText', ['$compile', function($compile){
        return {
            restrict: 'A',
            link : function($scope, $ele, $attrs) {
                var tag = $ele.tag();
                var ngModel = $attrs['ngModel'];
                var listModel = $attrs['listModel'];

                var html = '';
                html += tag == 'input' ?
                    '<input ng-if="%ngModel%" ng-model="%ngModel%" class="form-control" type="text"/>' :
                    '<textarea ng-if="%ngModel%" ng-model="%ngModel%" class="form-control"></textarea>';
                html += '<div class="zoc mb-5" ng-if="!%ngModel%">';
                html += '   <label class="btn btn-default btn-sm btn-checker ph10 mr5 mb5" ng-repeat="item in %listModel%" ng-class="{\'active\': item == %ngModel%, \'btn-info\': item == %ngModel%}" title="{{item}}">';
                html += '       <input type="radio" name="password" ng-model="%ngModel%" value="{{ item }}" class="hidden">';
                html += '       <span>{{item}}</span>';
                html += '       <span class="checker zoom08"><i class="icon glyphicon glyphicon-ok"></i></span>';
                html += '   </label>';
                html += '</div>';

                html = Landers.utils.str.replace(html, '%ngModel%', ngModel);
                html = Landers.utils.str.replace(html, '%listModel%', listModel);
                $ele.replaceWith($compile(html)($scope));
            }
        }
    }]);
;angular.module('Landers.angular')
    .factory('AngularApi', ['$httpParamSerializer', 'Ajax', function ($httpParamSerializer, Ajax) {
        function Api(config) {//api_url, url_style, url_params, ajax
            var url_styles = {
                restful : {
                    index: '{apiurl}?page={page}&pagesize={pagesize}',
                    create: '{apiurl}/create',
                    store: '{apiurl}',
                    show: '{apiurl}/{id}',
                    edit: '{apiurl}/{id}/edit',
                    update: '{apiurl}/{id}',
                    delete: '{apiurl}/{id}',
                    opposite: '{apiurl}/opposite/{id}/{prop}',
                    custom : '{apiurl}/{action}'
                },
                parallelism : {
                    index: '{apiurl}/list?page={page}&pagesize={pagesize}',
                    create: '{apiurl}/create',
                    store: '{apiurl}/create',
                    show: '{apiurl}/show?id={id}',
                    edit: '{apiurl}/edit?id={id}',
                    update: '{apiurl}/update?id={id}',
                    delete: '{apiurl}/delete?id={id}',
                    custom: '{apiurl}/{action}'
                },
                lwap : {
                    index: '{apiurl}?act=_loadlist_&page={page}&pagesize={pagesize}',
                    store: '{apiurl}?act=_create_',
                    show: '{apiurl}?act=_show_&id={id}',
                    edit: '{apiurl}?act=_loadedit_&id={id}',
                    update: '{apiurl}?act=_modify_&id={id}',
                    delete: '{apiurl}?act=_delete_&id={id}',
                    opposite : '{apiurl}?act=_opposite_&id={id}&field={field}',
                    custom: '{apiurl}?act={action}'
                }
            };

            var that = this;
            var ajax = config.ajax || Ajax.make();
            this.config = function(){
                if (arguments.length) {
                    var arg0 = arguments[0];
                    if ( typeof(arg0) == 'string' ) {
                        return config[arg0];
                    } else {
                        config = arguments[0];
                    }
                } else {
                    return config;
                }
            },
            this.debug = function(){
                ajax.debug();
                return this;
            },
            this.reset = function(action, api_url){
                url_styles[config.url_style][action] = '{apiurl}/' + api_url;
                return this;
            },
            this.buildUrl = function(action, params) {
                var replaces = angular.extend({}, params);
                replaces['apiurl'] = config.api_url;
                replaces['params'] = (function(){
                    var params = angular.extend({}, params || {}, config.url_params || {});
                    return $httpParamSerializer(params);
                })();
                var apiurl = url_styles[config.url_style][action];
                var link = apiurl.indexOf('?') > 0 ? '&' : '?';

                if (replaces['params']) apiurl += link + '{params}';
                for (var k in replaces) {
                    apiurl = apiurl.replace('{' + k + '}', replaces[k]);
                }
                return apiurl;
            },
            this.loading = function(is){
                ajax.loading(is);
                return this;
            },
            this.show = function(id, params){
                var url = this.buildUrl('show', {id:id});
                return ajax.get(url, params);
            },
            this.list = function(page, params, pagesize){
                var url = this.buildUrl('index', {page:page || 1, pagesize: pagesize ||''});
                return ajax.get(url, params);
            },
            this.edit = function(dat_id, params){
                var url = this.buildUrl('edit', {id:dat_id});
                return ajax.get(url, params);
            },
            this.create = function(data, params){
                var url = this.buildUrl('store');
                return ajax.post(url, data, params);
            },
            this.update = function(dat_id, data, params){
                var url = this.buildUrl('update', {id:dat_id});
                if ( config.url_style == 'restful' ) {
                    return ajax.put(url, data, params);
                } else {
                    return ajax.post(url, data, params);
                }
            },
            this.delete = function(dat_ids, data){
                data = data || {};
                data.ids = dat_ids;

                var params = {};
                if (angular.isArray(data.ids)) {
                    params.id = data.ids[0];
                } else {
                    params.id = data.ids;
                }

                var url = this.buildUrl('delete', params);

                if ( config.url_style == 'restful' ) {
                    return ajax.delete(url, data);
                } else {
                    return ajax.delete(url, data);
                }
            },
            this.opposite = function(dat_id, prop) {
                var url = this.buildUrl('opposite', {id:dat_id, prop:prop});
                return ajax.get(url);
            },
            this.get = function(action, params) {
                var url = this.buildUrl('custom', {action:action});
                return ajax.get(url, params);
            },
            this.post = function(action, data, params) {
                var url = this.buildUrl('custom', {action:action});
                return ajax.post(url, data, params);
            };
        }

        return {
            make : function(config){
                return new Api(config);
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('listSort', ['Flat', '$compile', function (Flat, $compile) {
        return {
            restrict: 'AE',
            // template: '<a ng-class="{dropup:isUp,hidden:isHidden}"><i class="caret"></i></a>',
            link: function ($scope, $ele, $attrs) {
                var model_key = $attrs['modelKey'];
                var sort_by = $attrs['sortBy'];

                var app = Flat.get($scope, model_key + '.$angularApp');
                if ( !app || !app.model) return;

                var by_key = [model_key, '$sort', 'key'].join('.');
                var hidden_condition1 = by_key + '!=' + "'" + sort_by + "'";
                var by_mode = [model_key, '$sort', 'mode'].join('.');
                var dropup_condition = by_mode + '==' + "'asc'";
                var dropdown_condition = by_mode + '==' + "'desc'";
                var ng_class1 = [
                    'hidden:' + hidden_condition1,
                    'dropup:' + dropup_condition,
                    'dropdown:' + dropdown_condition
                ].join(',');

                var hidden_condition2 = by_key + '==' + "'" + sort_by + "'";
                var ng_class2 = [
                    'hidden:' + hidden_condition2,
                ].join(',');

                var html = '';
                html += '<a ng-class="{' + ng_class1 + '}"><i class="caret"></i></a>';
                html += '<span ng-class="{' + ng_class2 + '}" style="display:inline-block; width:10px; height:12px; position:relative; color:#999;"><span class="dropup" style="position:absolute; left:0px; top:0px; display:inline-block; height:6px; "><i class="caret" style="margin-top:-14px;"></i></span><span class="dropdown" style="position:absolute; left:0px; bottom:0px; display:inline-block; height:6px "><i class="caret" style="margin-top:-10px;"></i></span></span>';

                var $eles = $($compile(html)($scope)).replaceAll($ele);

                $eles.css({'cursor': 'pointer'}).bind('click', function() {
                    app.model.trigger('sort', sort_by);
                });
            }
        }
    }])
    .provider('AngularModel', function() {
        var provider = this;

        this.$get = ['$templateRequest', 'List', 'Flat', 'AngularApi', 'Helpers', '$rootScope', '$timeout', '$state',  function ($templateRequest, List, Flat, AngularApi, Helpers, $rootScope, $timeout, $state) {
            var is_api_result = function(rspn) {
                    if ( !angular.isObject(rspn)) return false;
                    return  'success' in rspn &&
                            'message' in rspn &&
                            'data' in rspn;
            };

            function Model(scope, config) { // key, name, api_url, url_style, url_params, [ajax, lister, vice]

                var that = this, watchers = [];

                //定义模型标识
                if (!config.key) throw ('未定义key字段');

                //将当前模型数据容器
                var modelkey = '$' + config.key;
                this.modelkey = modelkey;
                this.primaryKey = config.primaryKey;

                scope[modelkey] = {
                    '$name' : config.name,
                    '$list' : {
                        'status': null,
                        'data' : null,
                        'page' : null
                    },
                    '$create' : null,
                    '$detail' : null,
                    '$items' : {
                        selected:[],
                    },
                    '$choosedId': null,
                    '$edit' : {},
                    '$form' : {},
                    '$update' : null,
                    '$modify' : null,
                    '$search' : null,
                    '$sort': '',
                    // {
                    //     // key:'id',
                    //     // mode:'desc'
                    // },
                    '$errors' : {
                        '$list' : null,
                        '$edit' : null,
                        '$form' : null,
                    },
                    '$runtime' : {},
                    '$event': {},
                    '$enums': {},
                    '$angularModel': this,
                    '$angularApp': null,
                    '$isCurrent': config.is_current
                };

                this.trigger = function(){
                    var fun, event, args = [];
                    for (var i=0; i<arguments.length; i++ ) {
                        if ( i== 0) {
                            event = arguments[i];
                        } else {
                            args.push(arguments[i]);
                        }
                    }
                    var fun = scope[modelkey].$event[event];

                    if ( fun ) {
                        fun.apply(scope[modelkey].$event, args)
                    }
                };

                this.getDataKey = function( key, subkey ) {
                    return modelkey + '.' + key + (subkey ? '.' + subkey : '');
                };

                this.getConfig = function(key) {
                    return key ? config[key] : config;
                };

                this.extendData = function(key, extdata, subkey) {
                    var data = that.getData(key, subkey);
                    Flat.extend(data, extdata);
                    angular_scope_apply(scope);
                    return this;
                };

                this.setData = function(key, data, subkey) {
                    var datakey = modelkey + '.' + key;
                    Flat.set(scope, datakey, data, subkey);
                    angular_scope_apply(scope);
                    return this;
                };

                this.setDataToString = function(keys, subkey) {
                    if ( !angular.isArray(keys) ) keys = [keys];
                    keys.map(function(key){
                        var value = that.getData(key, subkey);
                        that.setData(key, value.toString(), subkey);
                    })
                };

                this.getData = function(key, subkey) {
                    var datakey = modelkey + '.' + key;
                    return Flat.get(scope, datakey, subkey);
                };

                this.watchers = {};
                this.watch = function(data_key, callback, is_append) {
                    if ( angular.isArray(data_key) ) {
                        angular.forEach(data_key, function(item){
                            that.watch(item, callback);
                        });
                        return null;
                    }

                    // 非追加时，要销毁之前的监听
                    if (!is_append) {
                        that.destroyWatchers(data_key);
                    }

                    if (angular.isString(data_key)) {
                        data_key = modelkey + '.' + data_key;
                    }

                    var watcher = scope.$watch(data_key, callback);
                    that.watchers[data_key] = watcher;
                    return that;
                };
                this.destroyWatchers = function(data_keys, watchers){
                    if ( data_keys ) {
                        if ( angular.isString(data_keys) ) {
                            data_keys = [data_keys];
                        }
                    } else {
                        data_keys = Object.keys(that.watchers);
                    }

                    data_keys.map(function(data_key){
                        data_key = modelkey + '.' + data_key;
                        if ( angular.isFunction(that.watchers[data_key]) ) {
                            that.watchers[data_key]();
                        }
                    });

                    if (angular.isArray(watchers)) {
                        watchers.map(function(fun){
                            if ( angular.isFunction(fun) ) {
                                fun();
                            }
                        });
                    }
                };

                this.clearErrors = function(key) {
                    that.setData('$errors', {}, key);
                };

                this.setErrors = function(key, data, message) {
                    for (var k in data) {
                        if (angular.isArray(data[k])) {
                            data[k] = data[k][0];
                        }
                    }
                    that.setData('$errors', data, key);

                    var isMsg;
                    if ( message === true || message === false) {
                        isMsg = message;
                        message = '提交的数据还存在不足';
                    } else {
                        isMsg = true;
                    }

                    if (isMsg && message) Landers.response.msgbox({
                        success : false,
                        message : message
                    });

                    return message;
                };

                //定义钩子模块
                // 即将改名： hooks 改名为 hook
                this.hook = this.hooks = {
                    data : {
                        list:{
                            before:[],
                            after:[]
                        },
                        edit:{
                            onload:{
                                before:[],
                                after:[]
                            },
                            submit:{
                                before:[],
                                after:[]
                            }
                        },
                        close:[]
                    },
                    registe: function(key, fns) {
                        var _this = this;
                        if (!angular.isArray(fns)) fns = [fns];
                        angular.forEach(fns, function (fn){
                            if (angular.isFunction(fn)) {
                                Flat.push(_this.data, key, fn);
                            }
                        });
                    },
                    registeOnce: function(key, fn) {
                        if ( Flat.get(this.data, key).length ) return;
                        this.registe( key, fn);
                    },
                    run : function(key, params){
                        var _this = this;
                        var funs = Flat.get(_this.data, key);
                        angular.forEach(funs, function (fn){fn(params);});
                    }
                };

                //创建api与服务端通信对象
                config['api_url'] = config['api_url'] || ('/' + config.key);
                this.api = AngularApi.make(Landers.utils.json.slice(config, [
                    'api_url', 'url_style', 'url_params', 'ajax'
                ]));

                //定义列表操作对象
                // laravel输出分页时，数据位于 data字段中
                var lister = config.lister || List.make(scope, modelkey + '.$list.data', null);
                lister.setIdKey(that.primaryKey);
                this.lister = lister;

                //列表操作模块
                this.list = {
                    put:function(data) {
                        return that.lister.put(data);
                    },
                    get:function(){
                        return that.lister.all();
                    },
                    clear:function(){
                        that.listitem.unselectAll();

                        var current = that.getData('$list.page.current');
                        that.extendData('$list', {
                            status: status || 'LOADING',
                            page: {current:current},
                            data: null
                        });

                        that.setData('$errors.$list', null);
                        return this;
                    },
                    doShow:function(ajax, opts){
                        opts = opts || {};
                        return ajax.success(function(rspn){
                            if (opts.handleResponse) {
                                opts.handleResponse(rspn);
                            }

                            if (opts.show !== false) {
                                if (is_api_result(rspn)) {
                                    Landers.response.msgbox(rspn);
                                    var data = rspn.data;

                                    // data 部分是数组 (laravel不分页时)
                                    if ( angular.isArray(data) ) {
                                        that.setData('$list', {
                                            data: rspn.data,
                                            page: null
                                        });
                                    }

                                    // lwap返回格式
                                    if ( data.$list ) {
                                        data.$list.data = data.$list.data || [];
                                        data.$list.page = data.$list.page || null;
                                        that.setData('$list', data.$list);
                                    }

                                    // laravel分页返回格式
                                    if ( data.data ) {
                                        that.setData('$list', {
                                            data: data.data,
                                            page: {
                                                current: data.current_page,
                                                last: data.last_page,
                                                size: data.per_page,
                                                total: data.total
                                            }
                                        });
                                    }
                                } else {
                                    if ( angular.isArray(rspn) ) {
                                        //RESTFUL 返回纯数据
                                        that.setData('$list.data', rspn);
                                    }
                                }
                            }

                            that.hooks.run('list.after', rspn);
                            opts.onShow && opts.onShow(rspn);
                            that.setData('$list.status', 'COMPLETE');
                        }).error(function(rspn){
                            try {
                                that.setData('$errors.$list', rspn.message);
                                that.setData('$list',
                                    {data : [], page:{}, 'status': 'COMPLETED'}
                                );
                            } catch (e) {}
                        });
                    },
                    _params:{},
                    show:function(opts){
                        var _this = this;
                        opts = opts || {};
                        that.hooks.run('list.before');

                        if (opts.api) {
                            that.api.reset('index', opts.api);
                            that.api.reset('list', opts.api);
                        }

                        var $search = angular.extend({},that.getData('$search') || {}, opts.search || {});

                        delete $search.temp;
                        for (var k in $search) {
                            // if ( k.indexOf('$$$') === 0 ) delete $search[k];
                            if ( $search[k] === '') delete $search[k];
                        }

                        var $sort = that.getData('$sort');
                        if ($sort && angular.isString($sort)) {
                            $sort = Landers.utils.str.replace($sort, '\'', '"');
                            $sort = angular.fromJson($sort);
                        }
                        $sort = angular.extend({}, $sort || {});

                        angular.extend(this._params, opts.params || {});
                        var params = angular.extend({}, this._params, {
                            search: angular.toJson($search),
                            sort: angular.toJson($sort)
                        });
                        var doShow = function() {
                            var ajax = that.api.loading(opts.loading)
                                .list(opts.page, params, opts.pagesize || config.pagesize);
                            return _this.doShow(ajax, opts);
                        };

                        if (opts.show !== false) this.clear();
                        if (opts.callbackBefore) {
                            return opts.callbackBefore(doShow);
                        } else {
                            return doShow();
                        }
                    },

                    refresh:function(callback){
                        this.clear();
                        var opts = {
                            page: that.getData('$list.page.current') || 1,
                            params: {
                                search:angular.toJson(that.getData('$search'))
                            },
                            onShow: callback
                        };
                        var _this = this;
                        $timeout(function(){
                            _this.show(opts);
                        }, 200);
                    },
                    ifRefresh : function(){
                        if (!lister.count()) this.refresh();
                    },
                    preAppend : function(data){
                        return lister.preAppend(data);
                    },
                    append : function(data) {
                        return lister.append(data);
                    }
                };

                //列表横操作模块
                this.listitem = {
                    each: function(callback) {
                        lister.item.each(callback);
                    },
                    put: function(data){
                        if (!data) return false;
                        var list = that.getData('$list.data');
                        var index = this.mapIndex(data);
                        if (index >= 0) {
                            list[index] = data;
                            return true;
                        } else {
                            return false;
                        }
                    },
                    data: function(primary_key) {
                        return lister.item.data(primary_key);
                    },
                    mapById: function(ids){
                        return lister.item.mapById(ids);
                    },
                    mapIndex: function(mix) {
                        return lister.item.mapIndex(mix);
                    },
                    mapByIndex: function(indexs) {
                        return lister.item.mapByIndex(indexs);
                    },

                    showDeleteConfirm: function($event, item, item_name){
                        Helpers.disable($event);
                        var unactive = this.active($event);

                        if (item) item.$busy = true;
                        var what_name = item_name ? ( config.name + '【' + item[item_name] + '】') : ('该' + config.name);
                        if (!confirm('确定要删除' + what_name + '吗？')) {
                            unactive($event);
                            Helpers.enable($event);
                            if (item) item.$busy = false;
                            return false;
                        } else {
                            return true;
                        }
                    },
                    show:function(id, params){
                        return that.api.show(id, params)
                            .success(function(rspn){
                                that.setData('$detail', rspn.data);
                            })
                    },
                    edit:function(id, params){
                        return that.api.edit(id, params)
                            .success(function(rspn){
                                that.setData('$edit', rspn.data);
                            })
                    },
                    swap:function(item1, item2){
                        return that.api.get('swap-sorter/' + item1.id + '/' + item2.id)
                            .success(function(){
                                lister.item.swap(item1, item2);
                            })
                            .error(function(rspn){
                                Landers.response.msgbox(rspn);
                            });
                    },
                    moveUp:function(item1) {
                        var index2, index1 = this.mapIndex(item1);
                        if (index1 === 0) {
                            return alert('当前项已经处于最前了');
                        } else {
                            index2 = index1 - 1;
                        }
                        var item2 = lister.item.getByIndex(index2);
                        this.swap(item1, item2);
                    },
                    moveDown:function(item1){
                        var index2, index1 = this.mapIndex(item1);
                        if (index1 === lister.count() - 1 ) {
                            return alert('当前项已经处于最后了');
                        } else {
                            index2 = index1 + 1;
                        }
                        var item2 = lister.item.getByIndex(index2);
                        this.swap(item1, item2);
                    },
                    update:function(item, data){
                        that.lister.item.update(item, data);
                    },
                    active:function($event, active_class){
                        lister.item.activeByEvent($event, active_class);

                        return function() {
                            setTimeout(function(){
                                that.listitem.unactive($event, active_class)
                            }, 100);
                        };
                    },
                    unactive: function($event, active_class){
                        return lister.item.unactiveByEvent($event, active_class);
                    },
                    select:function($event, is_check){
                        if (is_check) {
                            return lister.item.checkByEvent($event);
                        } else {
                            return lister.item.uncheckByEvent($event);
                        }
                    },
                    selectAll : function(bool, datkey){
                        datkey = datkey || '$list.data';
                        var data = that.getData(datkey);
                        if ( data ) {
                            data = bool ? data.map(function(item){
                                return item[that.primaryKey] || item.key;
                            }) : [];
                            that.setData('$items', data, 'selected');
                        }
                    },
                    unselectAll: function(){
                        return this.selectAll(false);
                    },
                    getSelected: function(fields){
                        var list = that.getData('$list.data');
                        var result = list.filter(function(item){
                            return item.checked;
                        });
                        if (fields) {
                            return result.map(function(item){
                                var _item = {};
                                fields.map(function(field){
                                    _item[field] = item[field];
                                });
                                return _item;
                            });
                        } else {
                            return result
                        }
                    },
                    getSelectedId: function(alert_message){
                        if (alert_message === true) {
                            alert_message = '未选择目标数据！';
                        }
                        var items = this.getSelected(['id']);
                        var ids = items.map(function(item){
                            return item.id;
                        });

                        if ((!ids || !ids.length) && alert_message) {
                            alert(alert_message);
                            throw alert_message;
                            return null;
                        }
                        return ids;
                    },
                    remove:function(ids, lister) {
                        var lister = lister || that.lister;

                        if (!angular.isArray(ids)) {
                            ids = [ids];
                        }
                        var selectIds = that.getData('$items', 'selected');
                        angular.forEach(ids, function(id){
                            lister.item.remove(id);
                            var index = selectIds.findIndex(function(_id){
                                return id === _id;
                            });
                            if (index >= 0) {
                                selectIds.splice(index, 1);
                            }
                        });

                        var total = that.getData('$list.page.total');
                        if (total) {
                            that.setData('$list.page.total', total - ids.length);
                        }
                    },
                    deleteById:function(ids, lister, params){
                        var _this = this;
                        return that.api.delete(ids, params || {})
                            .success(function(rspn){
                                _this.remove(ids, lister);
                                that.list.ifRefresh();
                            });
                    },
                    // 被 dev_ssetting 重写 deleteItem 调用时，需要传入lister
                    delete:function(item, lister, params){
                        return this.deleteById(item[that.primaryKey], lister, params)
                            .error(function(rspn){
                                item.$busy = false;
                            });
                    },
                    deleteSelected:function(callback_before, params){
                        var _this = this;
                        var ids = this.getSelectedId(true);
                        if (!ids) return;
                        if ( !confirm('确定要删除选择的数据？')) return;
                        var do_delete = function(){
                            return _this.deleteById(ids, null, params)
                                .success(function(rspn){
                                    Landers.response.msgbox(rspn);
                                    _this.unselectAll();
                                })
                                .error(function(rspn){
                                    Landers.response.msgbox(rspn);
                                })
                        };

                        if ( callback_before ) {
                            callback_before(function(){
                                return do_delete();
                            })
                        } else {
                            return do_delete();
                        }
                    }
                };

                this.addedit = {
                    doEdit: function(item, onLoad, params, hook){
                        return that.api.edit(item[that.primaryKey], params || {})
                           .success(function(rspn){
                                var editData = (function(){
                                    if ( !rspn.data ) return rspn;
                                    if ( rspn.data.$edit ) return rspn.data.$edit;
                                    return rspn.data;
                                })();
                                if (editData) that.setData('$edit', editData);
                                onLoad && onLoad(rspn);
                                hook && hook(rspn);
                           });
                    },
                    doUpdate: function($event, item, onSubmit, hook){
                        var data = that.getData('$edit');
                        return that.api.update(item[that.primaryKey], data)
                            .success(function(rspn){
                                var updated_data = null;

                                if ( is_api_result(rspn) ) {
                                    if ( rspn.data ) {
                                        updated_data = rspn.data.$modify || rspn.data.$update || rspn.data;
                                    }
                                } else {
                                    if ( angular.isObject(rspn) ) {
                                        updated_data = rspn;
                                    } else if (rspn === true) {
                                        updated_data = that.getData('$edit');
                                    }
                                }

                                if ( !updated_data ) {
                                    return that.list.refresh();
                                } else {
                                    that.listitem.update(item, updated_data);

                                    // 调用回调
                                    onSubmit && onSubmit(rspn);

                                    // 更新到模型缓存
                                    that.setData('$update', updated_data)

                                    // 调用钩子
                                    hook && hook(rspn);
                                }
                            });
                    },
                    doCreate: function(onSubmit, hook){
                        var data = that.getData('$edit');
                        return that.api.create(data)
                            .success(function(rspn){
                                var created_data = null;

                                var page_info = that.getData('$list.page') || {};
                                if ( !('$total' in  page_info) ) {
                                    if ( is_api_result(rspn) ) {
                                        if ( rspn.data ) {
                                            created_data = rspn.data.$create || rspn.data;
                                        }
                                    } else {
                                        if ( angular.isObject(rspn) ) {
                                            created_data = rspn;
                                        } else if (rspn === true) {
                                            created_data = that.getData('$edit');
                                        }
                                    }
                                }

                                if ( !created_data ) {
                                    return that.list.refresh();
                                } else {
                                    that.list.preAppend(created_data);

                                    // 调用回调
                                    onSubmit && onSubmit(rspn);

                                    // 更新到模型缓存
                                    that.setData('$create', created_data)

                                    // 调用钩子
                                    hook && hook(rspn);
                                }
                            });
                    },
                    show: function($event, item, callback){
                        var action = item ?
                            {key : 'edit', text : '编辑', is_edit : true, datId: item[that.primaryKey], event:$event} :
                            {key : 'create', text : '添加', is_edit : false, datId:null, event:$event };

                        var _this = this;

                        var doLoad = function(onLoad, params){
                            that.setData('$edit', {});
                            that.setData('$errors.$edit', null);
                            that.hooks.run('edit.onload.before', {action:action});
                            that.setData('$runtime.is_edit', action.is_edit);
                            if (action.is_edit) {
                                return _this.doEdit(item, onLoad, params, function(rspn){
                                    that.hooks.run('edit.onload.after', {action:action, rspn:rspn});
                                });
                            } else {
                                onLoad && onLoad({});
                                that.hooks.run('edit.onload.after', {action:action});
                            }
                        };
                        var doSubmit = function(onSubmit){
                            that.hooks.run('edit.submit.before', {action:action});
                            return  action.is_edit ?
                                    _this.doUpdate($event, item, onSubmit, function(rspn){
                                        that.hooks.run('edit.submit.after', {action:action, rspn:rspn});
                                    }) :
                                    _this.doCreate( onSubmit, function(rspn){
                                        that.hooks.run('edit.submit.after', {action:action, rspn:rspn});
                                    });
                        };

                        callback(action, doLoad, doSubmit);
                    }
                };

                this.enums = function(){
                    if ( arguments.length == 2) {
                        that.extendData('$enums.' + arguments[0], arguments[1]);
                    } else {
                        that.extendData('$enums', arguments[0]);
                    }
                };


                //新初始化$event, （注：必须不能用angular.extend来扩展，因为scope可能某子controller并继承了父controller）
                that.extendData('$event', {
                    // 用于外部扩展时方便取得 当前Model
                    getModel: function(){
                        return that;
                    },
                    sort: function(key){
                        var data = that.getData('$sort');
                        if ( data.key ) {
                            if ( data.key == key) {
                                data.mode = data.mode == 'asc' ? 'desc' : 'asc';
                            } else {
                                data = {key: key, mode:'asc'};
                            }
                        } else {
                            data = {key: key, mode:'asc'};
                        }
                        that.setData('$sort', data);
                        that.list.show();
                    },
                    selectAll : function($event, datakey) {
                        $event = angular.element($event.target);
                        var checked = $event.prop('checked');
                        that.listitem.selectAll(checked, datakey);
                    },
                    refreshList : function( callback ){
                        that.list.refresh( callback );
                    },
                    deleteItem : function($event, item, params, item_name) {
                        if ( angular.isString(params)) {
                            params = angular.fromJson(str_params);
                        }
                        params = params || {};
                        if ( !that.listitem.showDeleteConfirm($event, item, item_name) ) {
                            // 点击取消
                            return {
                                success:function(){
                                    //donoting
                                    return this;
                                },
                                error:function(callback){
                                    callback();
                                    return this;
                                },
                                finally : function(callback) {
                                    callback();
                                    return this;
                                }
                            };
                        } else {
                            // 点击确定
                            return that.listitem.delete(item, null, params)
                                .success(function(rspn){
                                    Landers.response.msgbox(rspn);
                                })
                                .error(function(rspn){
                                    Landers.response.msgbox(rspn);
                                    Helpers.enable($event);
                                })
                                .finally(function(){
                                    that.listitem.unactive($event);
                                });
                        }
                    },
                    deleteSelected : function() {
                        that.listitem.deleteSelected();
                    },
                    search : function(page){
                        var opts = {page : page};
                        var dat = that.getData('$search') || {};
                        if (dat._keyword_ || dat._keyword_by_) {
                            if (!dat._keyword_) {
                                return alert('未输入搜索关键词');
                            }
                            if (!dat._keyword_by_) {
                                return alert('未选择搜索关键词依据');
                            }
                        }
                        var ajax = that.list.show(opts);
                        if ( this.onSearch ) this.onSearch(ajax);
                        return ajax;
                    },
                    showPage : function(page){
                        var ajax = this.search(page);
                        if ( this.onShowPage ) this.onShowPage(ajax);
                    },
                    resetSearch : function(){
                        that.setData('$search', {});
                        scope.$emit('reset-search');
                        scope.$broadcast('reset-search');
                        that.list.show();
                    },
                    moveUp : function($event, item) {
                        that.listitem.moveUp(item);
                    },
                    moveDown : function($event, item) {
                        that.listitem.moveDown(item);
                    },
                    opposite : function($event, item, prop){
                        return that.api.opposite(item[that.primaryKey], prop)
                            .success(function(rspn){
                                if ( angular.isObject(rspn.data) ) {
                                    item[prop] = rspn.data[prop];
                                }
                                if (
                                    rspn.data === true ||
                                    rspn.data === 1 ||
                                    rspn.data === false ||
                                    rspn.data === 0
                                ) {
                                    item[prop] = rspn.data;
                                }
                            })
                            .error(function(rspn){
                                Landers.response.msgbox(rspn);
                            })
                    }
                });

                that.extendData('$enums', $rootScope.$enums || $rootScope.Enums || {});

                scope.$on('$destroy', function(){
                    that.destroyWatchers();
                    Landers.response.hide();
                });
            }

            return {
                make:function(scope, config){
                    return new Model(scope, config);
                },
                isApiResult: is_api_result,
            };
        }];
    });
;angular.module('Landers.angular')
    .directive('crudDoItem', function(){
        return {
            restrict : 'A',
            link : function($scope, $element, $attrs){
                $element.addClass('doitem')
                    .children('a, button').not('.btn').addClass('btn ')
                    .each(function(){
                        if ($(this).attr('class').indexOf('btn-') < 0 ) {
                            $(this).addClass('btn-primary');
                        }
                    })
            }
        }
    })
    .directive('subnav', ['$rootScope', function($rootScope){
        return {
            restrict: 'A',
            link: function($scope, $element, $attrs){
                // $scope.$on('clearSubnav', function(event){
                //     $element.empty()
                // });
                // $scope.$on('showSubnav', function(event, $subnav){
                //     $element.append($subnav);
                // });
                $scope.$emit('SetSubnavOjbect', $element);
            }
        }
    }])
    .directive('lockInput', ['$compile', '$timeout', 'Runtime', function($compile, $timeout, Runtime){
        return {
            restrict: 'A',
            link: function($scope, $element, $attrs){
                var get_lock_key = function() {
                    var lock_key = $attrs['lockInputKey'];
                    if (!lock_key) {
                        var ngModel = $attrs['ngModel'];
                        var arr = ngModel.split('.');
                        lock_key= arr[arr.length-1];
                    }
                    return lock_key;
                }

                var key = get_lock_key();
                var runtime = Runtime.make($scope);

                $scope.$watch(function(){
                    return $attrs['lockInputInit'];
                }, function(init){
                    if (init === undefined) return;
                    runtime.set('lockInput', init==='true', key);
                    var $wraper = $element.parent();
                    if ($wraper.find('.btn-lock-input').length) return;

                    var $input_group = $wraper.hasClass('input-group') ? $wraper : (function(){
                        var $ret = $('<div class="input-group"></div>').appendTo($wraper);
                        $ret.append($element);
                        return $ret;
                    }());

                    var $input_group_addon = $input_group.find('.input-group-addon');
                    if (!$input_group_addon.length) {
                        $input_group_addon = $('<div class="input-group-addon"></div>').appendTo($input_group);
                    }

                    var html = '';
                    html += '<a class="btn-lock-input lock-input-on" ng-if="$runtime.lockInput.' + key + '" href="javascript:;" ng-click="unlockInput(\'' + key + '\')">启用编辑</a>';
                    html += '<a class="btn-lock-input lock-input-on" ng-if="!$runtime.lockInput.' + key + '" href="javascript:;" ng-click="lockInput(\'' + key + '\')">禁用编辑</a>';
                    $input_group_addon.append($compile(html)($scope));
                });

                angular.extend($scope, {
                    unlockInput: function(lock_key){
                        runtime.set('lockInput', false, lock_key);
                    },
                    lockInput: function(lock_key){
                        runtime.set('lockInput', true, lock_key);
                    }
                });
            }
        }
    }])
    .provider('AngularApp', function () {
        var provider = this;

        this.cache = false;

        this.path = {
            module: null,
            component: null
        };

        this.extras = {};

        this.needCss = false;

        this.modulePath = function(module){
            return (provider.extras[module] || provider.path.module) + '/' + module;
        };

        provider.showDialogError = function(dialog, rspn) {
            if ( rspn.errors ) {
                var errors = Object.values(rspn.errors);

                if ( errors.length > 1) {
                    errors = errors.map(function(value, index){
                        return '<li>' + value + '</li>';
                    });
                    errors.splice(0, 0, '<ul>');
                    errors.push('</ul>')
                    rspn.message = errors.join('');
                    Landers.msgbox({
                        succes: false,
                        message : rspn.message
                    });
                    dialog.done();
                } else {
                    rspn.message = errors.join('');
                    dialog.done(rspn);
                }
            } else {
                Landers.msgbox({
                    succes: false,
                    message : rspn.message
                });
                dialog.done();
            }
        };

        provider.cache = {
            injectedApps: {}
        };

        this.$get = ['$state', 'Ajax', 'Runtime', 'AngularModel', 'Dialog', '$templateRequest', 'Flat', '$timeout', 'InjectService', '$compile', 'Helpers', '$rootScope', '$compile', function ($state, Ajax, Runtime, AngularModel, Dialog, $templateRequest, Flat, $timeout, InjectService, $compile, Helpers, $rootScope, $compile) {
            var defaults = {
                dialog : {
                    width : 800,
                    height : '100%',
                    scroll : true
                },
                partials: {

                }
            };

            function DynamicInjecte(scope) {
                var that = this;

                function parse_path_info(path) {
                    var a = path.split('/');
                    if ( a.length == 1) {
                        return {dir: path, file: ''} ;
                    } else {
                        var file = a[a.length-1];
                        a.length--;
                        return {
                            dir: a.join('/'),
                            file: file
                        };
                    }
                }

                this.getAppResourcesUrl = function(module){
                    var base_path = provider.modulePath(module);
                    return [
                        base_path + '/style.css',
                        base_path + '/client.js'
                    ];
                };
                // this.injectedModules = {};
                this.injectModule = function(module, injects){
                    // var modules = that.injectedModules[module];
                    // if (!modules) {
                        var urls = that.getAppResourcesUrl(module);
                        InjectService.module(urls, injects);
                    // } else {
                        // callback.apply(this, modules);
                    // }
                };
                this.injectedApps = {};
                this.bootApp = function(path, callback_controller) {
                    var path_info = parse_path_info(path);
                    var urls = that.getAppResourcesUrl(path_info.dir);
                    var callback = function(){
                        if (angular.isString(callback_controller)) {
                            that.bootController(callback_controller);
                        }
                        if (angular.isFunction(callback_controller)){
                            var mod_path = provider.modulePath(path_info.dir); // provider.path.module + '/' + path_info.dir;
                            var main_url = mod_path + '/' + (path_info.file || 'layout.html');
                            that.bootController(main_url);
                            $timeout(function(){
                                callback_controller(scope);
                            }, 200);
                        }
                        that.injectedApps[path] = true;
                    }

                    if (!that.injectedApps[path]) {
                        InjectService.urls(urls, callback);
                    } else {
                        callback();
                    }
                };
                this.injectedControllers = {};
                this.bootController = function (controller_url, reload){
                    if (!that.injectedControllers[controller_url] || reload) {
                        var html = controller_url.indexOf('/') >= 0 ?
                                   '<div ng-include="\'' + controller_url + '\'"></div>' :
                                   '<div ng-controller="' + controller_url + '"></div>';
                        var el = $compile(html)(scope);
                        angular.element('#base-body, .base-body').append(el);
                        angular_scope_apply(scope);
                        that.injectedControllers[controller_url] = true;
                    };
                };
                this.resetInjectCache = function() {
                    that.injectedControllers = {};
                    that.injectedModules = {};
                    that.injectedApps = {};
                };
            };

            var SubnavManager = {
                dom: null,
                hadInit: function(){
                    return !!this.dom
                },
                init: function($dom){
                    this.dom = $dom;
                    return this;
                },
                clear: function(){
                    if (this.dom) {
                        this.dom.empty();
                    };
                    return this;
                },
                show: function(scope, partial){
                    var dom = this.dom;
                    if ( dom && dom.length ) {
                        dom.empty();
                        if ( partial ) {
                            if ( partial === true ) {
                                partial = 'subnav';
                            }
                            var partial_url = scope.$partial( partial );
                            $templateRequest(partial_url).then(function(html){
                                dom.append( $($compile(html)(scope)) );
                                scope.$emit('SetSubnav', dom);
                            });
                        }
                    }
                }
            };

            function AngularApp(scope, config) {
                config = $.extend(true, {}, defaults, config);

                //初始化全局变量
                var that = this,
                    doms = [];

                this.scope = scope;

                // 即将废除
                scope.$APPS = scope.$APPS || {};
                scope.$APPS[config.key] = this;

                this.primaryKey = config.primary_key || 'id';

                //初始化 key, name
                this.key = config.key;
                this.name = config.name;

                this.isCurrent = config.is_current || ($state.current.url + '/').indexOf(config.api_url + '/') === 0;

                //初始化path
                this.path = provider.modulePath(config.path || that.key);

                if ( !scope.hasOwnProperty('$partial') ) {
                    scope.$partial = function(partial, path){
                        if (path === undefined && config.partials) {
                            path = config.partials[partial];
                            if ( path ) return path;
                        }

                        if ( !path ) {
                            path = that.path;
                        } else if ( path === true ) {
                            path = provider.path.component;
                        } else {
                            if ( provider.extras[path] ) {
                                path = provider.modulePath(path);
                            } else {
                                path = that.path + '/../' + path;
                            }
                        }

                        var html_url = path + '/partials/' +  partial + '.html';
                        html_url = Landers.url.real(html_url);

                        // if ( !provider.cache ) html_url += '?_=' + Math.random();
                        return html_url;
                    };
                }

                //加载css
                if (provider.needCss) {
                    var css_url = that.path + '/style.css';
                    if ( !provider.cache ) css_url += '?_=' + Math.random();
                    doms.push(Landers.loader.css(css_url));
                }

                //初始化ajax
                this.ajax = Ajax.make();

                //Subnav
                if (that.isCurrent) {
                    if ( SubnavManager.hadInit() ) {
                        SubnavManager.show(scope, config.subnav);
                    } else {
                        $rootScope.$on('SetSubnavOjbect', function(event, $subnav){
                            SubnavManager.init($subnav).show(scope, config.subnav);
                        });
                    }
                }

                //初始化runtime (挂载$runtime至scope)
                this.runtime = Runtime.make(scope);
                this.runtime.set('referer', document.referrer);
                this.runtime.set('$app', this); // 即将废除
                if (Flat.get($state, '$current.data.app')) {
                    this.runtime.set('$current', $state.$current.data.app);
                    if (that.isCurrent) this.runtime.set('$current.app', this);
                }

                if ( config.ctrl ) {
                    this.ctrl = angular.extend(config.ctrl, Flat.make(config.ctrl));
                }

                this.$ = {
                    _datkey : function(objectkey, subkey){
                        var datkey = ['$' + objectkey];
                        if (subkey) datkey.push(subkey);
                        return datkey.join('.');
                    },
                    get : function(objectkey, subkey) {
                        var datkey = this._datkey(objectkey, subkey);
                        return Flat.get(scope, datkey);
                    },
                    set : function(objectkey, data, subkey) {
                        var datkey = this._datkey(objectkey);
                        Flat.set(scope, datkey, data, subkey);
                        angular_scope_apply(scope);
                    },
                    extend : function(objectkey, data) {
                        var datkey = this._datkey(objectkey);
                        Flat.extend(scope, data, datkey);
                        angular_scope_apply(scope);
                    }
                };

                //对话框
                this.dialog = {
                    dialogs : [],
                    _attachConfig : function(_dialog_config){
                        var dialog_config = angular.extend({}, _dialog_config || {});
                        if (_dialog_config.event) {
                            if (that.model) {
                                that.model.listitem.active(_dialog_config.event);
                            }
                            dialog_config.onclose = function(){
                                if (that.model) {
                                    that.model.listitem.unactive(_dialog_config.event);
                                }
                                _dialog_config.onclose && _dialog_config.onclose.call(this);
                            };
                            delete dialog_config.event;
                        }
                        if ( _dialog_config.bottombarPartial ) {
                            var bottombar_url = scope.$partial( _dialog_config.bottombarPartial, _dialog_config.partial_path );
                            var bottombar_class = dialog_config.bottombarClass || '';
                            dialog_config.bottombar = '<div ng-include src="\'' + bottombar_url + '\'" class="' + bottombar_class + '"></div>';
                        }

                        if ( _dialog_config.statusbarPartial ) {
                            var bottombar_url = scope.$partial( _dialog_config.statusbarPartial, _dialog_config.partial_path );
                            var statusbar_class = dialog_config.statusbarClass || '';
                            dialog_config.statusbar = '<div ng-include src="\'' + bottombar_url + '\'" class="' + statusbar_class + '"></div>';
                        }
                        // dialog_config.blur = false;
                        return dialog_config;
                    },
                    _loadPartial : function(dialog_config, callback){
                        var partial_url;
                        if ( dialog_config.partial ) {
                            if (angular.isArray(dialog_config.partial)) {
                                partial_url = scope.$partial( dialog_config.partial[0], dialog_config.partial[1] );
                            } else {
                                if (
                                    dialog_config.partial.indexOf('/') === 0 ||
                                    dialog_config.partial.indexOf('../') === 0
                                ) {
                                    partial_url = dialog_config.partial;
                                } else {
                                    partial_url = scope.$partial( dialog_config.partial, dialog_config.partial_path );
                                }
                            }
                        } else {
                            throw ('参数错误');
                        }
                        $templateRequest(partial_url).then(function(html){
                            callback(html);
                        });
                    },

                    show : function(_dialog_config){
                        var dialog_config = this._attachConfig(_dialog_config);
                        var _this = this;
                        var _show = function(){
                            if (_dialog_config.timeout!==false) {
                                _dialog_config.timeout = 30;
                            }
                            if (_dialog_config.timeout) {
                                $timeout(function(){
                                    var dialog = Dialog.show(scope, dialog_config);
                                    _this.dialogs.push(dialog);
                                }, _dialog_config.timeout);
                            } else {
                                console.log('no no');
                                var dialog = Dialog.show(scope, dialog_config);
                                _this.dialogs.push(dialog);
                            }
                        };
                        if (dialog_config.partial) {
                            _this._loadPartial(dialog_config, function(html){
                                dialog_config.html = html;
                                _show();
                            });
                        } else {
                            _show();
                        }
                    },

                    // 传入要显示的内容和按钮的回调
                    confirm : function(mix, callbacks, $event){
                        var _this = this;
                        var dialog_config;
                        if ( angular.isString(mix) ) {
                            // 处理content内容
                            var content = mix.replace(/&/g, "&amp;")
                                .replace(/</g, "&lt;")
                                .replace(/>/g, "&gt;");
                            dialog_config = {
                                content: '<div class=\"confirm\">' + content + '</div>'
                            };
                        } else if (angular.isObject(mix)) {
                            dialog_config = angular.extend({}, mix);
                        }
                        dialog_config = angular.extend({
                            title: '来自'+window.location.host+'的消息',
                            width: 300,
                            event: $event
                        }, dialog_config);

                        dialog_config = this._attachConfig(dialog_config);

                        if ( dialog_config.partial ) {
                            _this._loadPartial(dialog_config, function(html){
                                dialog_config.html = html;
                                var dialog = Dialog.confirm(scope, dialog_config, callbacks);
                                _this.dialogs.push(dialog);
                            });
                        } else {
                            var dialog = Dialog.confirm(scope, dialog_config, callbacks);
                            this.dialogs.push(dialog);
                        }
                    }
                };

                //初始化model
                this.appkey = '$' + config.key;

                if (config.model !== false) {
                    this.model = AngularModel.make(scope, {
                        key : that.key,
                        primaryKey: that.primaryKey,
                        name : that.name,
                        is_current: that.isCurrent,
                        path: {

                        },
                        api_url: (function(){
                            if ( config.api_url ) {
                                return config.api_url;
                            }
                            if ( config.url_style == 'lwap') {
                                return that.path + '/server.php';
                            }
                            return '/' + config.key;
                        })(),
                        url_style : config.url_style,
                        url_params : (function(){
                            var parent_source = config.parent_source;
                            var ret = config.url_params || {};
                            //新参数parent_source
                            if (parent_source && parent_source.app) {
                                ret.related_key = parent_source.app.key;
                                if ( parent_source.listitem ) {
                                    ret.related_id = parent_source.listitem[that.primaryKey];
                                }
                            };
                            //兼容老参数parentApp
                            if (config.parentApp && config.parentApp.prop) {
                                ret.appid = config.parentApp.prop.appid;
                                if ( config.parentEvent ) {
                                    ret.datid = config.parentApp.model.listitem.data(config.parentEvent, 'id');
                                }
                            };
                            return ret;
                        })(),
                        ajax: that.ajax,
                        vice: config.vice,
                        pagesize: config.pagesize,
                        template_urls: config.template_urls
                    });

                    //model.list.popup
                    this.model.list.popup = function(_dialog_config, url_params, callback){
                        var _this = this; url_params = url_params || {};
                        var dialog_config = that.dialog._attachConfig(_dialog_config);
                        dialog_config.onload = function(){
                            var dialog = this.done();
                            var ajax = _this.show({params: url_params});
                            callback && callback(ajax);
                            _dialog_config.onload && _dialog_config.onload.call(dialog);
                            setTimeout(function(){dialog.position();}, 300);
                        };

                        dialog_config = angular.extend({}, {
                            title: that.name + '列表',
                            partial : dialog_config.partial || 'lister',
                            scroll : true,
                            width: 600,
                            height : '100%',
                            no : true
                        }, dialog_config);
                        that.dialog.show(dialog_config);
                    };

                    this.model.detail = {
                        show: function(item, params) {
                            return that.model.listitem.show(item[that.primaryKey], params);
                        },
                        popup: function(item, _dialog_config, params) {
                            var _this = this;
                            var dialog_config = that.dialog._attachConfig(_dialog_config);
                            dialog_config.onload = function() {
                                var dialog = this.done();
                                _this.show(item, params)
                                    .error(function(rspn) {
                                        dialog.done(rspn);
                                    })
                                    .finally(function(){
                                        _dialog_config.onload && _dialog_config.onload.call(dialog);
                                        setTimeout(function(){dialog.position();}, 300);
                                    });
                            };

                            dialog_config = angular.extend({}, {
                                partial: dialog_config.partial || 'detail',
                                scroll: true,
                                loading: {size:'small'},
                                height: '100%',
                                no: true
                            }, dialog_config);
                            that.dialog.show(dialog_config);
                        }
                    };

                    this.model.addedit.popup = function($event, item, _dialog_config) {
                        var dialog_config = that.dialog._attachConfig(_dialog_config);
                        dialog_config.partial = dialog_config.partial || dialog_config.tplkey || 'form';

                        var _this = this, globalDialog;
                        that.dialog._loadPartial(dialog_config, function(dialog_html){
                            _this.show($event, item, function (action, doLoad, doSubmit){
                                if (action.is_edit) {
                                    if (dialog_config.loading === undefined) {
                                        dialog_config.loading = {size:'small'};
                                    }
                                    dialog_config.blur = false;
                                }
                                dialog_config = angular.extend({}, dialog_config, {
                                    event : $event,
                                    title : action.text + (dialog_config.name || that.name),
                                    html: dialog_html,
                                    onload : function(){
                                        globalDialog = this.done();
                                        var onBeforeLoad = dialog_config.onBeforeLoad;
                                        var _onAfterLoad = function(rspn){
                                            var onAfterLoad = dialog_config.onAfterLoad;
                                            onAfterLoad && onAfterLoad.call(that, action, globalDialog, rspn);

                                            if ( AngularModel.isApiResult(rspn) ) globalDialog.done(rspn); else globalDialog.done();
                                        };
                                        var _doLoad = function(){
                                            var ajax = doLoad(_onAfterLoad);
                                            if (ajax) ajax.error(function(rspn){
                                                globalDialog.done(rspn);
                                            });
                                        };
                                        $timeout(function() {
                                            if ( onBeforeLoad ) {
                                                onBeforeLoad(action, globalDialog, _doLoad);
                                            } else {
                                                _doLoad();
                                            }
                                        }, 100);
                                    },
                                    yes : {
                                        text:'保存', fn:function(window, button){
                                            Helpers.disable($(button));
                                            var onBeforeSubmit = dialog_config.onBeforeSubmit;
                                            var onAfterSubmit = dialog_config.onAfterSubmit;
                                            var _doSubmit = function(){
                                                doSubmit()
                                                    .success(function(rspn){
                                                        Helpers.enable($(button));
                                                        onAfterSubmit && onAfterSubmit(action, globalDialog, rspn, item);
                                                        if (!rspn.message) {
                                                            globalDialog.done().close();
                                                        } else {
                                                            if ( action.is_edit ) {
                                                                globalDialog.done(rspn).btn().get('取消').text('关闭');
                                                            } else {
                                                                globalDialog.done().close();
                                                                Landers.response.msgbox(rspn);
                                                            }
                                                        }
                                                    })
                                                    .error(function(rspn){
                                                        if ( angular.isObject(rspn.errors) ) {
                                                            var message = that.model.setErrors('$edit', rspn.errors, false);
                                                            rspn.message = rspn.message || message;
                                                        }
                                                        provider.showDialogError(globalDialog, rspn);
                                                        Helpers.enable($(button));
                                                    });
                                            };

                                            globalDialog.loading({
                                                text : '正在提交数据...',
                                                size : 'long'
                                            }, function(callback_result){
                                                $timeout(function() {
                                                    if ( onBeforeSubmit ) {
                                                        onBeforeSubmit(action, globalDialog, _doSubmit);
                                                    } else {
                                                        _doSubmit();
                                                    }
                                                }, 100);
                                            });
                                            return false;
                                        }
                                    },
                                    no : '取消'
                                });
                                dialog_config.onclose = dialog_config.onclose || function(){
                                    scope.$broadcast('destroy_ueditor');
                                    that.model.setData('$edit', {});
                                };
                                that.dialog.show(dialog_config);
                            });
                        });
                    };

                    this.action = {
                        loading:function(){
                            if (that.ajax) that.ajax.loading();
                            return this;
                        },
                        buildUrl: function(action){
                            var apiurl = that.model.api.buildUrl('custom', {action:action || ''});
                            if ( !action ) {
                                var arr = apiurl.split('/');
                                if ( !arr[arr.length-1] ) {
                                    arr.length--;
                                }
                                apiurl = arr.join('/');
                            }
                            return apiurl;
                        },
                        post : function(action, data, params) {
                            // var apiurl = that.model.api.buildUrl('custom', {action:action});
                            var apiurl = this.buildUrl(action);
                            if (!data && that.model) data = that.model.getData('$edit');
                            if (!that.ajax) throw ('未定义 ajax');
                            return that.ajax.post(apiurl, data, params);
                        },
                        get : function(action, params) {
                            // var apiurl = that.model.api.buildUrl('custom', {action:action});
                            var apiurl = this.buildUrl(action);
                            if (!that.ajax) throw ('未定义 ajax');
                            return that.ajax.get(apiurl, params);
                        },
                        put: function(action, data, params) {
                            var apiurl = this.buildUrl(action);
                            if (!that.ajax) throw ('未定义 ajax');
                            return that.ajax.put(apiurl, data, params);
                        },
                        'delete': function(action, data, params) {
                            var apiurl = this.buildUrl(action);
                            if (!that.ajax) throw ('未定义 ajax');
                            return that.ajax.delete(apiurl, data, params);
                        }
                    };

                    this.appkey = this.model.modelkey;

                    this.model.setData('$angularApp', this);
                }

                this.injectClient = function(name, injects){
                    var url = that.path + '/' + 'client-' + name + '.js';
                    InjectService.module([url], injects);
                };

                this.service = {
                    tagsInput : function(fields, prefix){
                        if (!angular.isArray(fields)) fields = [fields];
                        prefix = prefix || '$edit';
                        angular.forEach(fields, function(item){
                            var value = that.model.getData(prefix, item);
                            var is_array = angular.isArray(value);
                            that.runtime.set(item, {
                                data : value ? value.toString().split(',') : [],
                                onChange : function(){
                                    var value = this.data.map(function(item){
                                        return item.text;
                                    });
                                    if (!is_array) value = value.join(',');
                                    that.model.setData(prefix, value, item);
                                }
                            });
                        });
                    }
                };

                //扩展或覆盖$event
                if ( that.model ) {
                    that.model.extendData('$event', {
                        // 用于外部扩展时方便取得 当前App
                        getApp: function(){
                            return that;
                        },
                        showEdit:function($event, item){
                            $timeout(function() {
                                config.dialog.name = config.name;
                                config.dialog.partial = config.dialog.partial || 'form';
                                that.model.addedit.popup($event, item, config.dialog);
                            }, 100);
                        },
                        showCreate:function($event){
                            this.showEdit($event);
                        }
                    });
                    that.model.extendData('$event', config.events || {});

                } else {
                    Flat.set(scope, that.appkey, config.events, '$event');
                    angular_scope_apply(scope);
                }

                //初始化
                // scope.$errors = {};

                //scope销毁时事件
                scope.$on('$destroy', function(event){
                    config.onDestroy && config.onDestroy(event);

                    //todo: 要调用下面 return对象中的reset
                    that.resetInjectCache();

                    //断开所有连接
                    that.ajax.cancelAll();

                    //清除doms
                    angular.forEach(doms, function(dom){
                        dom.remove();
                    });

                    // Landers.dialog.clear && Landers.dialog.clear();
                    angular.forEach(that.dialog.dialogs, function(dialog){
                        if (dialog && dialog.close) {
                            dialog.close();
                        }
                    });

                    Landers.response.hide();

                    delete that.ajax;
                    delete that.model;
                    delete that.runtime;
                    delete that.$;
                    delete that.scope;
                });

                this.__proto__ = new DynamicInjecte(that.scope);
            }

            function attach_scope_utils_for_ui(scope)
            {
                scope.util = {
                    arrayPush: function(key, dat) {
                        Flat.push(scope, key, dat);
                    },
                    arraySplice: function(key, index, dat) {
                        Flat.splice(scope, key, index, dat)
                    }
                };
            }

            return {
                make : function(scope, config){
                    attach_scope_utils_for_ui(scope);
                    return new AngularApp(scope, config);
                },
                graft: function( scope, mod_name ){
                    return Flat.get(scope, ['$APPS', mod_name].join('.'));
                },
                partial: function(path, partial) {
                    return provider.modulePath(path) + '/partials/' +  partial + '.html';
                },
                provider: provider,
                launch:function(key) {
                    $state.go('base.' + key);
                },
                __proto__: new DynamicInjecte
            };
        }];
    });
;angular.module('Landers.angular')
    .factory('EchartsService', function(){
        var planePath = 'path://M1705.06,1318.313v-89.254l-319.9-221.799l0.073-208.063c0.521-84.662-26.629-121.796-63.961-121.491c-37.332-0.305-64.482,36.829-63.961,121.491l0.073,208.063l-319.9,221.799v89.254l330.343-157.288l12.238,241.308l-134.449,92.931l0.531,42.034l175.125-42.917l175.125,42.917l0.531-42.034l-134.449-92.931l12.238-241.308L1705.06,1318.313z';
        // var color = ['#a6c84c', '#ffa022', '#46bee9'];
        var curveness = 0.2;

        var that;

        return {
            init : function(id){
                that = that || this;

                if (!window.echarts) throw '未加载echarts库';
                var dom = document.getElementById(id);
                if (!dom) throw '未找到dom对象：' + id;

                var chart = echarts.init(dom);
                $(window).on('resize', chart.resize);

                return chart;
            },

            destroy : function(chart) {
                $(window).off('resize', chart.size);
                chart.dispose();
            },
            map : {
                getPosByGeo : function(chartInstance, point, serie_index) {
                    var seriesModel = chartInstance.getModel();
                    if (seriesModel) {
                        var series = seriesModel.getSeriesByIndex(serie_index);
                        var coordSys = series.coordinateSystem;
                        var coord = coordSys.dataToPoint([point.lng, point.lat]);
                        return {left : coord[0], top : coord[1]};
                    } else {
                        console.log('seriesModel 为空！');
                        return {left : 0, top : 0};
                    }
                },
                getGeoByPos : function(chartInstance, coord, serie_index) {
                    var seriesModel = chartInstance.getModel();
                    if ( seriesModel ) {
                        var series = seriesModel.getSeriesByIndex(serie_index);
                        var coordSys = series.coordinateSystem;
                        var point = coordSys.dataToPoint([coord.x, coord.y]);
                        return {lng : point[0], lat : point[1]};
                    } else {
                        console.log('seriesModel 为空！');
                        return {lng : 0, lat : 0};
                    }
                },
                buildPath: function(from, tos) {
                    var ret = [];
                    angular.forEach(tos , function(to){
                        ret.push([{
                            coord: [to.lng, to.lat]
                        }, {
                            coord: [from.lng, from.lat]
                        }]);
                    });
                    return ret;
                },

                //轨迹
                track : function(name, dest, srcs) {
                    return {
                        name: name,
                        type: 'lines',
                        zlevel: 300,
                        effect: {
                            show: true,
                            period: 6,
                            trailLength: 0.1,
                            color: '#fff',
                            symbolSize: 3,
                        },
                        lineStyle: {
                            normal: {
                                // color: '#xxxxxx',
                                width: 0,
                                trailLength: 0.1,
                                curveness: curveness
                            }
                        },
                        data: that.map.buildPath(dest, srcs)
                    };
                },

                //线条
                line : function(dest, srcs, color) {
                    return {
                        name: dest.ip,
                        type: 'lines',
                        zlevel: 100,
                        effect: {
                            show: true,
                            period: 6,
                            trailLength: 0,
                            symbol: 'none',
                            symbolSize: 0
                        },
                        lineStyle: {
                            normal: {
                                color: color, //that.colors.src[type].line,
                                width: 1,
                                opacity: 0.1,
                                curveness: curveness
                            }
                        },
                        data: that.map.buildPath(dest, srcs)
                    };
                },

                //点
                dot : function(name, dots, color, zlevel) {
                    return {
                        name: name,
                        type: 'effectScatter',
                        coordinateSystem: 'geo',
                        legendHoverLink : true,
                        zlevel: zlevel,
                        rippleEffect: {
                            brushType: 'stroke'
                        },
                        label: {
                            normal: {
                                show: true,
                                position: 'right',
                                formatter: '{b}'
                            }
                        },
                        symbolSize: function (val) {
                            return val[2];
                        },
                        itemStyle: {
                            normal: {
                                color: color
                            }
                        },
                        data: dots.map(function (dot) {
                            return angular.extend({}, {
                                name: '',
                                // symbol : 'arrow',
                                value : [dot.lng, dot.lat, dot.level]
                            }, dot);
                        })
                    };
                },

                option : function(legends, series, opts){
                    opts = opts || {};
                    opts.legend = opts.legend || {};
                    return {
                        // backgroundColor: '#404a59',
                        title : {
                            text: '',
                            subtext: '',
                            left: 'center',
                            textStyle : {
                                color: '#fff'
                            }
                        },
                        tooltip : {
                            show : false,
                            trigger: 'item'
                            // formatter : function(item){
                            //     var data = item.data;
                            //     //只显示攻击目标的信息
                            //     if (data.kind == 'dest') {
                            //         var content = [];
                            //         content.push( '攻击流量：' + data.bps + 'Gbps');
                            //         content.push( '攻击报文：' + data.pps + 'pps');
                            //         return content.join('<br/>');
                            //     } else {
                            //         return null;
                            //     }
                            // }
                        },
                        legend: {
                            orient: 'vertical',
                            top: opts.legend.top,
                            bottom:opts.legend.bottom,
                            left: opts.legend.left,
                            right: opts.legend.right,
                            align: opts.legend.align,
                            // data:['北京 Top10', '上海 Top10', '广州 Top10'],
                            data: legends,
                            textStyle: {
                                color: '#fff'
                            },
                            selectedMode: 'single'
                        },
                        geo: {
                            map: 'world',
                            label: {
                                emphasis: {
                                    show: false
                                }
                            },
                            roam: true,
                            itemStyle: {
                                normal: {
                                    areaColor: '#323c48',
                                    borderColor: '#404a59'
                                },
                                emphasis: {
                                    areaColor: '#2a333d'
                                }
                            }
                        },
                        series: series
                    };
                }
            },

            // line : {
            //     option : function(categories, datas){
            //         return  {
            //             title: {
            //                 text: ''
            //             },
            //             tooltip : {
            //                 trigger: 'axis',
            //                 axisPointer : {            // 坐标轴指示器，坐标轴触发有效
            //                     type : 'shadow',        // 默认为直线，可选为：'line' | 'shadow'
            //                     animation: false
            //                 }
            //             },
            //             toolbox: {
            //                 feature: {
            //                     saveAsImage: {}
            //                 }
            //             },
            //             grid: {
            //                 left: '3%',
            //                 right: '4%',
            //                 bottom: '3%',
            //                 containLabel: true
            //             },
            //             xAxis : [
            //                 {
            //                     type : 'category',
            //                     boundaryGap : false,
            //                     data : categories
            //                 }
            //             ],
            //             yAxis : [
            //                 {
            //                     type : 'value'
            //                 }
            //             ],
            //             series :[
            //                 {
            //                     name:'',
            //                     type:'line',
            //                     // stack: '总量',
            //                     areaStyle: {normal: {}},
            //                     data:datas
            //                 }
            //             ]
            //         }
            //     }
            // },

            // bar : {
            //     option : function(categories, datas){
            //         return {
            //             color: ['#3398DB'],
            //             tooltip : {
            //                 trigger: 'axis',
            //                 axisPointer : {            // 坐标轴指示器，坐标轴触发有效
            //                     type : 'shadow'        // 默认为直线，可选为：'line' | 'shadow'
            //                 }
            //             },
            //             toolbox: {
            //                 feature: {
            //                     saveAsImage: {}
            //                 }
            //             },
            //             grid: {
            //                 left: '3%',
            //                 right: '4%',
            //                 bottom: '3%',
            //                 containLabel: true
            //             },
            //             xAxis : [
            //                 {
            //                     type : 'category',
            //                     data : categories,
            //                     axisTick: {
            //                         alignWithLabel: true
            //                     }
            //                 }
            //             ],
            //             yAxis : [
            //                 {
            //                     type : 'value'
            //                 }
            //             ],
            //             series : [
            //                 {
            //                     name:'',
            //                     type:'bar',
            //                     barWidth: '60%',
            //                     data:datas
            //                 }
            //             ]
            //         };
            //     }
            // }
        };
    });
;angular.module('Landers.angular')
    .factory('PromiseLoads', ['$q', function($q) {
        return {
            asyn:function(tasks, callback){
                var promises = [];
                angular.forEach(tasks, function(task){
                    var deferred = $q.defer();
                    var ajax = task(deferred);
                    if (ajax) ajax.success(function(){
                        deferred.resolve();
                    }).error(function(){
                        deferred.reject();
                    });
                    promises.push(deferred.promise);
                });
                $q.all(promises).then(function(){
                    // 曾在 initCreate() 调用无延时出过错
                    setTimeout(callback, 200);
                });
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('money', ['Flat', '$compile', '$filter', function(Flat, $compile, $filter){
        Landers.css.add('.cny:after {content:"￥"}');
        return {
            restrict : 'E',
            scope: {
                amount: '=',
            },
            link : function($scope, $ele, $attrs) {
                var ui_sref = $attrs['uiSref'];
                var unit = $attrs['unit'] || '';

                if ($scope.amount) {
                    $scope.amount = parseFloat(parseFloat($scope.amount).toFixed(2));
                }

                $scope.$watch('amount', function(new_value){
                    Flat.set($scope, 'money.unit', unit );
                    Flat.set($scope, 'money.a', !!ui_sref);
                    Flat.set($scope, 'money.span', !ui_sref);
                    var if_class = "{'" + $attrs['class'] + "': amount>=0}";

                    var html = '';
                    html += '<span>';
                    html += '   <em class="cny" ng-show="money.span"></em>';
                    html += '   <span class="amount" ng-if="money.a" ng-class="' + if_class+ '" ng-bind-html="amount"></span>';
                    html += '   <span class="amount" ng-if="money.span" ng-class="' + if_class+ '" ng-bind-html="amount"></span>';
                    html += '   <em class="unit-cny" ng-show="money.unit">{{money.unit}}</em>';
                    html += '</span>';

                    $ele.replaceWith($compile(html)($scope));
                });
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('buttonRadio', ['$timeout', 'Flat', 'Helpers',  function($timeout, Flat, Helpers){
        return {
            restrict : 'A',
            link : function($scope, $group, $attrs) {
                var myScope = Helpers.seekScope($scope, $attrs['useScope']);
                var on_watch = $attrs['buttonRadioOnWatch'];
                var ngModel = $attrs['ngModel'];
                if ( on_watch ) {
                    on_watch = Flat.get($scope, on_watch);
                    $scope.$watch(ngModel, function(new_value){
                        if ( new_value === undefined ) return;
                        on_watch(new_value);
                    });
                }

                var doActive = function($button){
                    if ($button.hasClass('disabled') || $button.prop('disabled') || $button.attr('disabled')) {
                        return false;
                    }
                    var $input = $button.children();
                    Flat.set(myScope, ngModel, $input.val());

                    var a = ngModel.split('.');
                    a.splice(1, 0, '$errors');
                    var errorKey = a.join('.');
                    Flat.set(myScope, errorKey, '');

                    angular_scope_apply(myScope);

                    return true;
                };

                var ngModelList = $attrs['ngModelList'];
                if (ngModelList) {
                    $scope.$watch(ngModelList, function(value){
                        var $buttons = $group.children();

                        $buttons.unbind('click').click(function(){
                            return doActive($(this));
                        });

                    });
                } else {
                    (function(){
                        var $buttons = $group.children();
                        if ( !$buttons.length ) {
                            return setTimeout(arguments.callee, 300);
                        }

                        $buttons.click(function(){
                            return doActive($(this));
                        });
                    })();
                }
            }
        };
    }]);

;angular.module('Landers.angular')
    .directive('buttonCheckbox', ['$timeout', 'Flat', 'Helpers',  function($timeout, Flat, Helpers){
        var $event_button;
        return {
            restrict : 'A',
            link : function($scope, $group, $attrs) {
                var myScope = Helpers.seekScope($scope, $attrs['useScope']);
                var on_watch = $attrs['buttonCheckboxOnWatch'];
                if ( on_watch ) {
                    on_watch = Flat.get($scope, on_watch);
                }
                var ngModel = $attrs['ngModel'];
                var classActive = 'active';

                $scope.$watch(ngModel, function(values){
                    if ( !values ) return;
                    $timeout(function(){
                        var $inputs = $group.find('[type=checkbox]');
                        $inputs.each(function(){
                            var $input = $(this);
                            var val = $input.val();
                            var index = $.inArray(val.toString(), values);
                            if ( index >= 0) {
                                $input.parent().addClass(classActive);
                                $input.prop('checked', true)
                            } else {
                                $input.parent().removeClass(classActive);
                                $input.prop('checked', false)
                            }
                        });

                        if (on_watch) {
                            on_watch(values, $event_button);
                        }
                    }, 100)
                });

                var doActive = function($_event_button){
                    $event_button = $_event_button;
                    var $buttons = $group.children(), $input;
                    var values = [];

                    $timeout(function(){
                        $buttons.each(function(){
                            var $button = $(this);
                            if ($button.hasClass('disabled') || $button.prop('disabled') || $button.attr('disabled')) {
                                return;
                            }
                            $input = $button.children();
                            if ( $input.prop('checked') ) {
                                values.push($input.val());
                            }
                        });
                        Flat.set(myScope, ngModel, values);

                        var a = ngModel.split('.');
                        a.splice(1, 0, '$errors');
                        var errorKey = a.join('.');
                        Flat.set(myScope, errorKey, '');

                        angular_scope_apply(myScope);
                    }, 100);
                };

                var ngModelList = $attrs['ngModelList'];
                if (ngModelList) {
                    $scope.$watch(ngModelList, function(value){
                        var $buttons = $group.children();

                        $buttons.unbind('click').click(function(){
                            doActive($(this));
                        });
                    });
                } else {
                    (function(){
                        var $buttons = $group.children();
                        if ( !$buttons.length ) {
                            return setTimeout(arguments.callee, 300);
                        }

                        $buttons.click(function(){
                            doActive($(this));
                        });
                    })();
                }
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('copyText', [function(){
        Landers.loader.jsm('jquery.clipboard');
        return {
            restrict : 'A',
            link : function($scope, $ele, $attrs) {
                var fromKey = $attrs['copyTextFrom'];
                var $from = fromKey ? $(fromKey) : $();
                (function(){
                    var text = $attrs['copyText'];
                    var value = $from.val() || $from.text();
                    text = text || value;
                    if ($.fn.zclip && text) {
                        $ele.zclip({copy: text});
                    } else {
                        setTimeout(arguments.callee, 300);
                    }
                })();
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('dataslider', ['Flat', '$compile', 'Helpers', '$timeout', function(Flat, $compile, Helpers, $timeout){
        if (!Landers.slider) {
            Landers.loader.jsm(['landers.slider,true']);
        }
        return {
            restrict : 'E',
            link : function($scope, $ele, $attrs) {
                // var myScope = Helpers.seekScope($scope, $attrs['useScope']);
                var myScope = $scope;

                var configKey = $attrs['config'];
                var ngModel = $attrs['ngModel'];
                var slideStop = $attrs['slideStop'];
                var slideDisabled = $attrs['ngDisabled'];

                var id = $attrs['sliderId'];
                var infoKey = $attrs['info'];

                var html = '';
                html += '<div class="slider-wrapper">';
                html += '   <div data-config="{{ ' + configKey + ' }}" class="data_slider">';
                html += '        <input ng-model="' + ngModel + '" type="hidden"/>';
                html += '   </div>';
                html += '</div>';
                $ele = $($compile(html)(myScope)).replaceAll($ele);

                var destroyWatch;
                var $slider = $ele.find('.data_slider');
                if ( id ) $slider.attr('id', id);

                (function(){
                    var self1 = arguments.callee;

                    //设置监听
                    if (!destroyWatch) {
                        destroyWatch = myScope.$watch(ngModel, function(newValue){
                            if ( newValue !== undefined ) {
                                (function(){
                                    var self2 = arguments.callee;
                                    $slider.find('input').val(newValue);
                                    try {
                                        $slider.slider('option', 'value', newValue)
                                               .trigger('slide', {value:newValue});
                                    } catch (e){
                                        setTimeout(self2, 200);
                                    }
                                })();
                            }
                        });
                        $ele.find('.ui-slider').attachWatcher(destroyWatch);
                    }


                    myScope.$watch(configKey, function(newConfig){
                        if (!newConfig) return;
                        var config = newConfig;
                        // var config = Flat.get(myScope, configKey);

                        //config 有值了，Landers.slider才能正常执行
                        if ( Landers.slider && config ) {

                            config = angular.fromJson(config);
                            Flat.set(myScope, ngModel, config.value);

                            //必须延时，否则虽然config有值，但未反应到html中
                            $timeout(function(){
                                Landers.slider($slider, {
                                    info : infoKey ? '{{' + infoKey + '}}' : '',
                                    slidestop:function (event, ui){
                                        Flat.set(myScope, ngModel, ui.value);
                                        angular_scope_apply(myScope);

                                        if (slideStop) {
                                            var fun = Flat.get(myScope, slideStop);
                                            if (fun) {
                                                var arr = slideStop.split('.');
                                                arr.length--;
                                                var parent = arr.join('.');
                                                parent = Flat.get(myScope, parent);
                                                fun.call(parent, $slider);
                                            }
                                        }
                                    }
                                });
                                myScope.$watch(slideDisabled, function(newValue){
                                    if ( newValue !== undefined ) {
                                        if ( newValue ) {
                                            $slider.slider( 'disable' );
                                        } else {
                                            $slider.slider( 'enable' );
                                        }
                                    }
                                });
                            }, 200);

                        } else {
                            $timeout(self1, 300);
                        }
                    }, true);
                })();
            }
        };
    }]);
;angular.module('Landers.angular')
    // 建立web socket连接，对象存在rootScope下，方便全局调用
    // 监听web socket发来的消息，由rootScope进行广播
    .provider('GWebSocket', function(){
        var provider = this;
        this.connections = null;
        this.showLog = false;
        this.onOpen = null;

        // 初始化超时时间
        var initialTimeout = 1500;

        // 设置是否断开自动重连
        var reconnectIfNotNormalClose = true;

        // 所有的连接
        var connections = {};

        // var is_api_result = function(rspn) {
        //         if ( !angular.isObject(rspn)) return false;
        //         return  'success' in rspn &&
        //                 'message' in rspn &&
        //                 'status_code' in rspn &&
        //                 'response_code' in rspn &&
        //                 ('data' in rspn || 'errors' in rspn);
        // };

        this.$get = ['$rootScope','$websocket', function($rootScope, $websocket) {
            if (!provider.connections) {
                return;
            };

            var build_event = function(connection, cmd){
                return [connection, cmd].join(':');
            };

            var initWs = function(connection, ws_url){
                var ws = $websocket(ws_url);

                // 设置断开自动重连
                ws.reconnectIfNotNormalClose = reconnectIfNotNormalClose;

                // 设置初始化连接超时时间
                ws.initialTimeout = initialTimeout;

                // 设置创建连接时的回调函数，发送认证信息到服务器
                ws.onOpen(function () {
                    provider.onOpen && provider.onOpen();
                });

                // 设置回调函数，收到消息后向全局广播
                ws.onMessage(function (msg) {
                    if (msg && msg.data) {
                        var result = angular.fromJson(msg.data);
                        var handler = result.cmd || result.handler;
                        var data = result.data;
                        var event = build_event(connection, handler);
                        show_log('receive message：', event, data);

                        // 广播至各个scope
                        $rootScope.$broadcast(event, data);
                    }
                });

                ws.onClose(function() {
                    show_log('websocket ' + connection + ' is closed', '', '');
                });

                return ws;
            };

            var watcher = {
               key: '$WebsocketListers',
               init: function(scope, connection){
                   if (!scope.hasOwnProperty(this.key)) {
                       scope[this.key] = {};
                   }
                   if (!scope[this.key][connection]) {
                        scope[this.key][connection] = {};
                   }
               },
               get: function(scope, connection, cmd) {
                   this.init(scope, connection);
                   return scope[this.key][connection][cmd];
               },
               add: function(scope, connection, cmd, on_result) {
                   this.init(scope, connection);
                   scope[this.key][connection][cmd] = on_result;
               },
               remove: function(scope, connection, cmd) {
                   this.init(scope, connection);
                   delete scope[this.key][connection][cmd];
               }
           };

            var send = function(connection, scope, cmd_handler, data, lisener){
                //cmd:命令格式统一为类名_方法名，例如user_login
                //handler:命令格式统一为类名_方法名，例如user_login
                //data:json对象包
                var pack = {
                    cmd: cmd_handler,
                    data: data,
                    handler: cmd_handler
                };
                if ( lisener && !watcher.get(scope, connection, cmd_handler)) {
                    var event = build_event(connection, cmd_handler);
                    var ret = scope.$on(event, lisener);
                    watcher.add(scope, connection, cmd_handler, ret);
                }
                return connections[connection].ws.send(angular.toJson(pack));
            };

            var show_log = function(msg, cmd, data) {
                if (provider.showLog) {
                    console.log(msg, cmd, data);
                }
            };

            for (var connection in provider.connections) {
                connections[connection] = connections[connection] || (function(url){
                    var ws_protocol = window.location.protocol.substring(0,5) == 'https' ? 'wss' : 'ws';
                    var ws_url = ws_protocol + '://' + url;
                    return {
                        ws: initWs(connection, ws_url),
                        authed: false,
                        onOpen:　null
                    };
                })(provider.connections[connection]);
            }

            return {
                // 发送请求到服务端
                send: function(connection, scope, cmd, data, lisener){
                    if (!scope) return;
                    if ( connections[connection].authed || true) {
                        send(connection, scope, cmd, data, lisener)
                            .then(function () {
                                show_log('success to send message:', cmd, data);
                            }, function () {
                                // 发送失败的消息会被push到ws对象的sendQueue数组中
                                show_log('failed to send message:', cmd, data);
                            });
                    } else {
                        var self = arguments.callee;
                        setTimeout(function(){
                            self(connection, scope, cmd, data, lisener);
                        }, 500);
                    }
                },

                cancel: function(connection, scope, cmd){
                    var destroyWatch = watcher.get(scope, connection, cmd);
                    if ( destroyWatch ) {
                        destroyWatch();
                        watcher.remove(scope, connection, cmd);
                    }
                },

                auth : function(connection, handler, ws_token, callback) {
                    if ('authed' in connections[connection]) {
                        connections[connection].onOpen = function(){
                            send(connection, $rootScope, handler, {token: ws_token}, function(event, response){
                                connections[connection].authed = response.success;
                                callback && callback();
                            });
                        }
                    }

                    ['user_need_login', 'need_auth'].map(function(item){
                        var event = build_event(connection, item);
                        $rootScope.$on(event, function(){
                            connections[connection].onOpen();
                        });
                    });

                    connections[connection].onOpen();
                }
            };
        }];
    });
;angular.module('Landers.angular')
    .provider('GNotify', function(){
        var provider = this;
        provider.config = {
            startTop: 60,
            // templateUrl: '',
            position: 'right',
            duration: 10000
        };

        provider.setConfig = function(opts) {
            angular.extend(provider.config, opts || {});
        }

        provider.events = [];

        Landers.css.add('.notify-box {min-width: 200px}');

        provider.classes = ['notify-box', 'text-left', 'alert'];

        this.$get = ['$rootScope', 'notify', function($rootScope, notify) {
            var setConfig = function(opts){
                var config = angular.extend({}, provider.config, opts || {});
                notify.config(config);
            };

            return {
                show: function(rspn, opts){
                    if (!rspn.message) return;

                    setConfig(opts);

                    var classes = provider.classes.slice(0);
                    classes.push('p15 pt20');
                    if (!rspn.response_code) {
                        if (rspn.success) {
                            classes.push('cg-notify-success');
                        } else {
                            classes.push('cg-notify-danger');
                        }
                    } else {
                        switch(rspn.response_code) {
                            case 'success':
                                classes.push('cg-notify-success');
                                break;
                            case 'error':
                                classes.push('cg-notify-danger');
                                break;
                            case 'info':
                                classes.push('cg-notify-info');
                                break;
                        }
                    }

                    notify({
                        classes: classes.join(' '),
                        message: rspn.message
                    });
                },

                startlisten: function(){
                    var _this = this;
                    provider.events.map(function(event){
                        $rootScope.$on(event, function($event, rspn){
                            _this.show(rspn);
                        });
                    })
                }
            };
        }];
    });
;angular.module('Landers.angular')
    .directive('landersRaty', ['Flat', function(Flat){
        return {
            restrict : 'A',
            template : '<div class="raty-star center"></div><div class="raty-hint"></div>',
            link : function($scope, $ele, $attrs){
                var $star = $ele.find('.raty-star');
                var ngmodel = $attrs['ngModel'];
                var $hint = $ele.find('.raty-hint');
                var config = {};
                if ($attrs['ratyNumber']) config.number = ~~$attrs['ratyNumber'];
                if ($attrs['ratyScore']) config.score = ~~$attrs['ratyScore'];

                if ($attrs['ratyTargetType']) config.targetType = $attrs['ratyTargetType'];
                if ($attrs['ratyPath']) config.path = $attrs['ratyPath'];
                if ($attrs['ratySize']) config.size = ~~$attrs['ratySize'];
                config = angular.extend({}, {
                    number: 10, // 多少个星星设置
                    score: 10,   // 初始值是设置
                    targetType: 'number',   //类型选择，number是数字值，hint，是设置的数组值
                    path      : '/static/img/star/',
                    cancelOff : 'cancel-off-big.png',
                    cancelOn  : 'cancel-on-big.png',
                    size      : 24,
                    starHalf  : 'star-half-big.png',
                    starOff   : 'star-off-big.png',
                    starOn    : 'star-on-big.png',
                    cancel    : false,
                    targetKeep: true,
                    precision : false,//是否包含小数
                    click: function(score, evt) {
                        if (ngmodel) Flat.set($scope, ngmodel, score);
                    }
                }, config);
                if (ngmodel) Flat.set($scope, ngmodel, config.score);

                if (config.target === true) config.target = $hint;
                Landers.loader.jsm('jquery.raty', function(){
                    $star.raty(config);
                });
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('landersAttachment', ['$compile', 'Flat', 'Dialog', 'List', function($compile, Flat, Dialog, List) {
        return {
            restrict : 'A',
            link : function($scope, $element, $attrs){
                var model_key = $attrs['attachModel'];
                var view_key = $attrs['attachView'];

                var html = '';
                html += '<div class="relative">';
                html += '   <a ng-click="landersAttachmentAdd($event)" class="btn btn-sm btn-primary absolute right-top"><i class="glyphicon glyphicon-plus"></i></a>';
                html += '</div>';
                html += '<input ng-model="' + model_key + '" type="hidden"/>';
                html += '<ul class="hlist list">';
                html += '    <li ng-repeat="item in ' + view_key + '" class="mr20 list-item">';
                html += '        <a href="{{item.url}}" target="_blank">{{item.title}}</a>';
                html += '        <a ng-click="landersAttachmentRemove($event)" class="ml5 btn btn-warning btn-xs"><i class="glyphicon glyphicon-minus"></i></a>';
                html += '    </li>';
                html += '</ul>';
                var el = $compile(html)($scope);
                $element.css({height:'100%'}).append(el);
                angular_scope_apply($scope);
                $element.find('.list').css({'margin-right':$element.find('.btn').width() + 20 })

                var lister = List.make($scope, view_key).data([]);
                $scope.$watch(model_key, function(model){
                    if (model) lister.data(model);
                });
                var html = '';
                html += '<form class="form-horizontal">';
                html += '<input type="hidden" id="url" ng-model="$runtime.attachment.url"/>';
                html += '<table cellpadding="0" cellspacing="0" border="0" class="stereo form-table" style="width:520px;">';
                html += '    <tr>';
                html += '       <td><label class="control-label">标题</label></td>';
                html += '       <td><div class="control-input"><input type="text" class="form-control" ng-model="$runtime.attachment.title"/></div></td>';
                html += '    </tr>';
                html += '    <tr height="192">';
                html += '       <td><label class="control-label">上传</label></td>';
                html += '       <td><div class="control-input">';
                html += '            <div landers-upload class="upload text-center fl" rel="url" uploadid="7b2aa8e6ba1d06fae48d72ae28dc7469" tipid="upload_attachment_tip" pvw="180" pvh="180"></div>';
                html += '            <ul id="upload_attachment_tip" class="fl lh22"></ul>';
                html += '        </div></td>';
                html += '    </tr>';
                html += '</table>';
                html += '</form>';
                angular.extend($scope, {
                    landersAttachmentAdd : function(){
                        Flat.set($scope, '$runtime.attachment', {});
                        setTimeout(function(){
                            Dialog.show($scope, {
                                title   : '新增附件',
                                content : html,
                                top : '30%',
                                width   : 350,
                                lock    : true,
                                fixed   : true,
                                effect  : 'vmove2',
                                onload  : function(){
                                },
                                yes     : function(){
                                    var attach = Flat.get($scope, '$runtime.attachment');
                                    lister.append(angular.extend({}, attach));
                                    Flat.set($scope, model_key, angular.toJson(lister.data()));
                                },
                                no      : true
                            });
                        }, 200);
                    },
                    landersAttachmentRemove : function($event){
                        if (!confirm('确定要删除此附件？')) return;
                        lister.item.remove($event);
                        Flat.set($scope, model_key, angular.toJson(lister.data()));
                    }
                })
            }
        }
    }]);
;angular.module('Landers.angular')
    .directive('landersTab', ['$timeout', 'Helpers', function($timeout, Helpers) {
        return {
            restrict : 'A',
            scope: {
                'tabModel': '=?',
                'tabCloseEvent': '@',
                'tabOpenEvent': '@'
            },
            link : function($scope, $element, $attrs){
                $timeout(function(){
                    $element.itab({
                        active: $scope.tabModel,
                        onClose: function(tab_key){
                            if ($scope.tabCloseEvent) {
                                $scope.$emit($scope.tabCloseEvent, tab_key);
                            }
                        },
                        onOpen: function(tab_key){
                            $timeout(function(){
                                if ($scope.tabOpenEvent) {
                                    $scope.$emit($scope.tabOpenEvent, tab_key);
                                }
                                $scope.tabModel = tab_key;
                            }, 10);
                        }
                    });
                }, 100);
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('landersUpload', ['Flat', '$compile', 'Helpers', function(Flat, $compile, Helpers) {
        return {
            restrict : 'A',
            scope: {
                'ngModel': '=',
                'onUpload': '@',
                'onRemove': '@'
            },
            template: '<textarea class="wp100 h100 mb10" style="display:none"></textarea>',
            link : function($scope, $ele, $attrs) {
                var $textarea = $ele.find('textarea');
                var key = $attrs['uploadKey'];
                var options = Helpers.attrItemToJson($attrs, 'uploadOptions');

                // 给options附上事件
                angular.extend(options, {
                    onChange: function(data){
                        if (angular.isString(data)) {
                            data = angular.fromJson(data);
                        }
                        data = data || '';

                        $scope.ngModel = data;
                        angular_scope_apply($scope);
                    },
                    onUpload: $scope.onUpload,
                    onRemove: $scope.onRemove
                });

                var upload = new Landers.upload($ele, $textarea, key, options);
                var destroyWatch = $scope.$watch('ngModel', function(new_value){
                    if (new_value === undefined) return;
                    if (new_value) {
                        upload.setInput(new_value);
                    } else {
                        upload.clearValue();
                    }
                    upload.show();
                    destroyWatch();
                });
            }
        };
    }]);
;angular.module('Landers.angular')
    .directive('landersTreeSelect', ['$timeout', function($timeout) {
        return {
            restrict : 'A',
            require: 'ngModel',
            link : function($scope, $element, $attrs, ngModel){
                Landers.loader.jsm(['landers.treeselect'], function(){
                    var source = $attrs.source;
                    $scope.$watch(source, function(newValue){
                        $element.children().remove();
                        Landers.treeselect.select({
                            contain: $element, data:newValue, root:['', '选择分类']
                        });
                        var precache = $element.data('precache');
                        $element.val(precache);
                    });
                    angular_scope_apply($scope);

                    $scope.$watch($attrs['ngModel'], function(newValue){
                        $element.data('precache', newValue);
                        $element.val(newValue);
                    });
                });
            }
        };
    }]);

// /**
// Created by Dio on 17-9.
// http://inhu.net
//  */

(function() {
    "use strict";
    (function() {
        var NGUeditor, Editors = {};
        NGUeditor = angular.module("ng.ueditor", []);
        NGUeditor.directive("ueditor", ["$timeout", function($timeout) {
            return {
                restrict: "C",
                require: "ngModel",
                scope: {
                    config: "=",
                    ready: "="
                },
                link: function($S, element, attr, ctrl) {
                    var _NGUeditor, _updateByRender;
                    _updateByRender = false;
                    _NGUeditor = (function() {
                        function _NGUeditor() {
                            window.UEDITOR_CONFIG.serverUrl = '/ueditor';
                            this.bindRender();
                            this.initEditor();
                            return;
                        }

                        /**
                         * 初始化编辑器
                         * @return {[type]} [description]
                         */
                        _NGUeditor.prototype.delEditor = function(_editorId) {
                            if (UE.delEditor && Editors[_editorId]) {
                                Editors[_editorId].removeListener("contentChange");
                                UE.delEditor(_editorId);
                                Editors[_editorId] = null;
                                delete Editors[_editorId];
                                this.editor = null;
                            }
                        };

                        _NGUeditor.prototype.initEditor = function() {
                            var _UEConfig, _editorId, _self;
                            _self = this;
                            if (typeof UE === 'undefined') {
                                console.error("Please import the local resources of ueditor!");
                                return;
                            }
                            _UEConfig = $S.config ? $S.config : {};
                            _editorId = attr.id ? attr.id : "_editor" + (Date.now());
                            element[0].id = _editorId;
                            if (Editors[_editorId]) {
                                _self.delEditor(_editorId);
                            };
                            $timeout(function(){
                                _self.editor = new UE.ui.Editor(_UEConfig);
                                Editors[_editorId] = _self.editor;
                                _self.editor.render(_editorId);
                                _self.editor.ready(function() {
                                    _self.editorReady = true;
                                    _self.editor.addListener("contentChange", function() {
                                        ctrl.$setViewValue(_self.editor.getContent());
                                        if (!_updateByRender) {
                                            if (!$S.$$phase) {
                                                $S.$apply();
                                            }
                                        }
                                        _updateByRender = false;
                                    });
                                    if (_self.modelContent && _self.modelContent.length > 0) {
                                        _self.setEditorContent();
                                    }
                                    if (typeof $S.ready === "function") {
                                        $S.ready(_self.editor);
                                    }
                                    $S.$on('$destroy', function() {
                                        _self.delEditor(_editorId);
                                    });
                                    $S.$on('destroy_ueditor', function() {
                                        _self.delEditor(_editorId);
                                    });
                                });
                            }, 200);
                        };

                        _NGUeditor.prototype.setEditorContent = function(content) {
                            if (content == null) {
                                content = this.modelContent;
                            }
                            if (this.editor && this.editorReady) {
                                try {
                                    this.editor.setContent(content);
                                } catch (e) {
                                }
                            }
                        };

                        _NGUeditor.prototype.bindRender = function() {
                            var _self = this;
                            ctrl.$render = function() {
                                if (!ctrl.$isEmpty(ctrl.$viewValue)) {
                                    _self.modelContent = (ctrl.$isEmpty(ctrl.$viewValue) ? "" : ctrl.$viewValue);
                                    _updateByRender = true;
                                    _self.setEditorContent();
                                }
                            };
                        };

                        return _NGUeditor;

                    })();

                    if (typeof UE != 'undefined') {
                        new _NGUeditor();
                    } else {
                        window.UEDITOR_HOME_URL = '/static/components/ui_ueditor/';
                        Landers.loader.js(window.UEDITOR_HOME_URL + 'ueditor.all.min.js', function(){
                            new _NGUeditor();
                        })
                    }


                }
            };
        }]);
    })();

}).call(this);


// //# sourceMappingURL=angular-ueditor.js.map



// angular.module("ng.ueditor", [])
//       .directive("ueditor", [
//         '$rootScope',
//         function($rootScope) {
//             return {
//                 restrict: 'EA',
//                 require: 'ngModel',
//                 link: function(scope, ele, attrs, ctrl) {
//                     $rootScope.$emit('loading', '初始化编辑器...');//广播loading事件，可以删除
//                     var _self = this,
//                             _initContent,
//                             editor,
//                             editorReady = false,
//                             base = '/public/vendor/utf8_qiniu_ueditor-master', //ueditor目录
//                             _id = attrs.ueditor;
//                     var editorHandler = {
//                         init: function() {
//                             window.UEDITOR_HOME_URL = base + '/';
//                             var _self = this;
//                             if (typeof UE != 'undefined') {
//                                 editor = UE.getEditor(_id);
//                                 editor.ready(function() {
//                                     editor.setHeight(500);
//                                     editorReady = true;
//                                     $rootScope.$emit('loading', '');//编辑器初始化完成
//                                     editor.addListener('contentChange', function() {//双向绑定
//                                         if (!scope.$$phase) {
//                                             scope.$apply(function() {
//                                                 ctrl.$setViewValue(editor.getContent());
//                                             });
//                                         }
//                                     });
//                                 });
//                             }
//                             else {
//                                 Common.loadScript(base + '/ueditor.config.js', null);
//                                 Common.loadScript(base + '/ueditor.all.min.js', function() {
//                                     _self.init();
//                                 });
//                             }
//                         },
//                         setContent: function(content) {
//                             if (editor && editorReady) {
//                                 editor.setContent(content);
//                             }
//                         }
//                     };
//                     ctrl.$render = function() {
//                         _initContent = ctrl.$isEmpty(ctrl.$viewValue) ? '' : ctrl.$viewValue;
//                         editorHandler.setContent(_initContent);//双向绑定
//                     };
//                     editorHandler.init();
//                     //事件
//                     $rootScope.$on('$routeChangeStart', function() {
//                         editor && editor.destroy();
//                     });
//                 }
//             }
//         }
//     ]);

;angular.module('Landers.angular')
    .directive('breadcrumb', function() {
        var separator = '/';
       return {
            restrict: 'EA',
            template:
                '<ol class="breadcrumb">'+
                '   <li ng-repeat=\'bc in datasource\' ng-class="{\'active\':datasource.length-1 == $index}">'+
                '       <a ng-hide="datasource.length-1 == $index" ng-click="removeUntil($index)" ng-href="{{bc.href}}" class="hand">{{bc.label}}</a>'+
                '       <span ng-show="datasource.length-1 == $index">{{bc.label}}</span>'+
                '   </li>'+
                '</ol>',
            replace: true,
            scope: {
                    datasource: '=',
                    onActive: '&'
            },
            compile: function(tElement, tAttrs) {
                return function($scope, $elem, $attr) {
                    $scope.removeUntil = function(index) {
                        $scope.datasource = $scope.datasource.splice(0, index+1);
                        var onActive = $scope.onActive();
                        onActive && onActive(
                            $scope.datasource[index],
                            index
                        );
                        return false;
                    };

                };//end of return
            }//end of compile
       };
    });
    // module.directive('breadcrumb', ['$log', '$compile', function($log, $compile) {
    //     var separator = '/';
    //     var html = '';
    //     html += '   <ol class="breadcrumb">';
    //     html += '   <li ng-repeat=\'bc in datasource\' ng-class="{\'active\':datasource.length-1 == $index}">';
    //     html += '       <a ng-hide="datasource.length-1 == $index" ng-click="unregisterBreadCrumb($index)" ng-href="{{bc.href}}">'
    //     html += '           {{bc.label}}';
    //     html += '       </a>';
    //     html += '       <span ng-show="datasource.length-1 == $index">{{bc.label}}</span>';
    //     html += '   </li>';
    //     html += '</ol>'

    //    return {
    //         restrict: 'A',
    //         compile: function(tElement, tAttrs) {
    //             return function($scope, $elem, $attr) {


    //                 $ele = $($compile(html)(myScope)).replaceAll($ele);

    //                 buildPath = function() {
    //                         var p = "";
    //                         angular.forEach($scope.datasource, function(v) {
    //                            p = p + separator + v.label;
    //                         });
    //                         return p;
    //                     };

    //                 $scope.unregisterBreadCrumb = function(index) {
    //                     var newDs = $scope.datasource.slice();
    //                     $scope.datasource= [];
    //                     for(var i=0;i<index+1; i++){
    //                           $scope.datasource.push(newDs[i]);
    //                     }
    //                      var path = buildPath();
    //                      $scope.onNodeSelected()(path,index);
    //                 };

    //             };
    //         }
    //    };
    // }]);
